@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -89,7 +89,7 @@ let type_a : Ptree.decl =
     td_vis = Public;
     td_mut = false;
     td_inv = [];
-    td_wit = [];
+    td_wit = None;
     td_def = TDrange (c22, c46);
   }]
 
