@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -18,9 +18,10 @@ API calls
 
 (* BEGIN{buildenv} *)
 open Why3
-let config : Whyconf.config =
-  Whyconf.(load_default_config_if_needed (read_config None))
+let config : Whyconf.config = Whyconf.init_config None
 let main : Whyconf.main = Whyconf.get_main config
+let libdir = Whyconf.libdir main
+let datadir = Whyconf.datadir main
 let env : Env.env = Env.create_env (Whyconf.loadpath main)
 (* END{buildenv} *)
 
@@ -90,7 +91,7 @@ let d2 =
     let unit = Ident.id_fresh "unit" in
     Ity.create_pvsymbol unit Ity.ity_unit
   in
-  let def,rs = Expr.let_sym id
+  let def,_rs = Expr.let_sym id
       (Expr.c_fun [arg_unit] [] [post] Ity.Mxs.empty Ity.Mpv.empty body) in
   Pdecl.create_let_decl def
 
@@ -138,7 +139,7 @@ let alt_ergo : Whyconf.config_prover =
 
 let alt_ergo_driver : Driver.driver =
   try
-    Whyconf.load_driver main env alt_ergo.Whyconf.driver []
+    Whyconf.load_driver main env alt_ergo
   with e ->
     eprintf "Failed to load driver for alt-ergo: %a@."
       Exn_printer.exn_printer e;
@@ -150,12 +151,16 @@ let () =
       (fun i t ->
        let r =
          Call_provers.wait_on_call
-           (Driver.prove_task ~limit:Call_provers.empty_limit
-                              ~command:alt_ergo.Whyconf.command
-                              alt_ergo_driver t)
+           (Driver.prove_task
+              ~limit:Call_provers.empty_limit
+              ~libdir
+              ~datadir
+              ~command:alt_ergo.Whyconf.command
+              alt_ergo_driver
+              t)
        in
        printf "@[On task %d, alt-ergo answers %a@."
-              i (Call_provers.print_prover_result ~json_model:false) r;
+              i (Call_provers.print_prover_result ?json:None) r;
        i+1
       )
       1 my_tasks
