@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -17,6 +17,7 @@ type attribute = private {
   attr_string : string;
   attr_tag    : int;
 }
+[@@deriving sexp_of]
 
 module Mattr : Extmap.S with type key = attribute
 module Sattr : Extset.S with module M = Mattr
@@ -99,7 +100,7 @@ val id_user : ?attrs:Sattr.t -> string -> Loc.position -> preid
 val id_attr : ident -> Sattr.t -> preid
 
 (** create a duplicate pre-ident *)
-val id_clone : ?attrs:Sattr.t -> ident -> preid
+val id_clone : ?loc:Loc.position -> ?attrs:Sattr.t -> ident -> preid
 
 (** create a derived pre-ident (inherit attributes and location) *)
 val id_derive : ?attrs:Sattr.t -> string -> ident -> preid
@@ -165,8 +166,17 @@ val id_unique_attr :
 (** Do the same as id_unique except that it tries first to use
     the "name:" attribute to generate the name instead of id.id_string *)
 
+
+(** {2 General purpose attributes} *)
+
 val proxy_attr: attribute
 val useraxiom_attr: attribute
+val funlit: attribute
+
+val is_field_id_attr: attribute
+  (** indicates that the related ident is a field name, and its applications
+      should be printed in dotted notation *)
+
 
 (** {2 Attributes for handling counterexamples} *)
 
@@ -191,10 +201,31 @@ val create_written_attr: Loc.position -> attribute
     file_name is at the end for easier parsing (file_name can contain ":")
 *)
 
-val extract_written_loc: attribute -> Loc.position option
-(** Extract the location inside vc_written attribute. [None] if the attribute is
-    ill-formed.
-*)
+val get_written_loc: attribute -> Loc.position option
+(** Get the location inside vc_written attribute. [None] if the attribute is
+    ill-formed. *)
+
+val create_call_result_attr : Loc.position -> attribute
+(** A call result attribute on a counterexample model element marks the result
+    of a call at the given location *)
+
+val get_call_result_loc : attribute -> Loc.position option
+(** Get the call result location from an attribute. *)
+
+val has_rac_assume : Sattr.t -> bool
+(** Check if the attributes contain [[@RAC:assume]]. When a program annotation
+   is a conjunction, conjuncts marked by this annotation are added to the
+   preconditions when checking the programannotation during giant-step RAC. *)
+
+val create_call_id_attr_string : int -> string
+(** Create the string of an attribute of the form [[@RAC:call_id:<id>]]. *)
+
+val get_call_id_value : attribute -> int option
+(** Get the call id of the form [[@RAC:call_id:<id>]]. *)
+
+val search_attribute_value : (attribute -> 'a option) -> Sattr.t -> 'a option
+(** [search_attribute_value f attrs] applies f to the attributes in [attr] and
+    returns the first inhabitad result, if any, or [None] otherwise. *)
 
 val remove_model_attrs : attrs:Sattr.t -> Sattr.t
 (** Remove the counter-example attributes from an attribute set *)
@@ -233,7 +264,8 @@ val compute_model_trace_field: ident option -> int -> Sattr.t
     the built field attribute associated *)
 
 val extract_field: attribute -> (int * string) option
-(** Take an attribute and extract its depth, name if it was a field attribute *)
+(** Take an attribute and extract its depth, name if it was a field attribute
+    (["field:depth:field_name"]) *)
 
 val get_model_trace_attr : attrs:Sattr.t -> attribute
 (** Return an attribute of the form ["model_trace:*"].
@@ -242,3 +274,9 @@ val get_model_trace_attr : attrs:Sattr.t -> attribute
 val get_hyp_name: attrs:Sattr.t -> string option
 (** If attrs contains an attribute of the form ["hyp_name:<s>"] returns
     [Some <s>] or [None] if no attribute have this form *)
+
+val unused_suffix : string
+(** Suffix for unused variables kept for counterexample generation *)
+
+val unused_attr : attribute
+(** attribute for unused variables kept for counterexample generation *)
