@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -30,16 +30,13 @@ type interface_map = interface Mid.t
 type interface_export_map = interface Mid.t
 type blacklist = string list
 
-type 'a pp = Pp.formatter -> 'a -> unit
-
 type field_info = {
   field_name: string;
   field_trace: string;
   field_ident: ident option;
 }
 
-type printer_mapping = {
-  lsymbol_m          : string -> Term.lsymbol;
+type printing_info = {
   vc_term_loc        : Loc.position option;
   vc_term_attrs      : Sattr.t;
   queried_terms      : Term.term Mstr.t;
@@ -50,28 +47,25 @@ type printer_mapping = {
   set_str            : Sattr.t Mstr.t
 }
 
-let list_projs pm =
-  Wstdlib.Mstr.union (fun _ x _ -> Some x) pm.list_projections pm.list_fields
+let fields_projs pm =
+  Wstdlib.Mstr.union (fun _ x _ -> Some x) pm.list_fields pm.list_projections
 
 type printer_args = {
-  env        : Env.env;
-  prelude    : prelude;
-  th_prelude : prelude_map;
-  blacklist  : blacklist;
-  mutable printer_mapping : printer_mapping;
+  env           : Env.env;
+  prelude       : prelude;
+  th_prelude    : prelude_map;
+  blacklist     : blacklist;
+  printing_info : printing_info option ref;
 }
 
-type printer = printer_args -> ?old:in_channel -> task pp
+type printer = printer_args -> ?old:in_channel -> task Pp.pp
 
-type reg_printer = Pp.formatted * printer
-
-let printers : reg_printer Hstr.t = Hstr.create 17
+let printers : (Pp.formatted * printer) Hstr.t = Hstr.create 17
 
 exception KnownPrinter of string
 exception UnknownPrinter of string
 
-let get_default_printer_mapping = {
-  lsymbol_m = (function _ -> raise Not_found);
+let default_printing_info = {
   vc_term_loc = None;
   vc_term_attrs = Sattr.empty;
   queried_terms = Mstr.empty;
@@ -95,7 +89,7 @@ let list_printers () =
 
 let () = register_printer
   ~desc:"Dummy@ printer@ with@ no@ output@ (used@ for@ debugging)." "(null)"
-  (fun _ ?old:_ _ _ -> ())
+  (fun _ ?old:_ -> Pp.nothing)
 
 (** Syntax substitutions *)
 
@@ -329,7 +323,7 @@ let syntax_range_literal ?(cb=None) s fmt c =
           None
       in
       if base = 10 then begin
-          if BigInt.lt v BigInt.zero then fprintf fmt "-";
+          if BigInt.lt v BigInt.zero then pp_print_string fmt "-";
           Number.print_in_base base digits fmt (BigInt.abs v)
         end
       else
