@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -275,7 +275,7 @@ let rec descend vml t = match t.t_node with
       find vs vml
   | _ -> Trm (t, vml)
 
-let t_compare trigger attr loc t1 t2 =
+let t_compare ~trigger ~attr ~loc ~const t1 t2 =
   let comp_raise c =
     if c < 0 then raise CompLT else if c > 0 then raise CompGT in
   let perv_compare h1 h2 = comp_raise (Pervasives.compare h1 h2) in
@@ -331,7 +331,7 @@ let t_compare trigger attr loc t1 t2 =
           | Tvar v1, Tvar v2 ->
               comp_raise (vs_compare v1 v2)
           | Tconst c1, Tconst c2 ->
-              comp_raise (Constant.compare_const c1 c2)
+              comp_raise (Constant.compare_const ~structural:const c1 c2)
           | Tapp (s1,l1), Tapp (s2,l2) ->
               comp_raise (ls_compare s1 s2);
               List.iter2 (t_compare bnd vml1 vml2) l1 l2
@@ -400,7 +400,7 @@ let t_similar t1 t2 =
   oty_equal t1.t_ty t2.t_ty &&
   match t1.t_node, t2.t_node with
     | Tvar v1, Tvar v2 -> vs_equal v1 v2
-    | Tconst c1, Tconst c2 -> Constant.compare_const c1 c2 = 0
+    | Tconst c1, Tconst c2 -> Constant.compare_const ~structural:true c1 c2 = 0
     | Tapp (s1,l1), Tapp (s2,l2) -> ls_equal s1 s2 && Lists.equal (==) l1 l2
     | Tif (f1,t1,e1), Tif (f2,t2,e2) -> f1 == f2 && t1 == t2 && e1 == e2
     | Tlet (t1,bv1), Tlet (t2,bv2) -> t1 == t2 && bv1 == bv2
@@ -412,7 +412,7 @@ let t_similar t1 t2 =
     | Ttrue, Ttrue | Tfalse, Tfalse -> true
     | _, _ -> false
 
-let t_hash trigger attr t =
+let t_hash ~trigger ~attr ~const t =
   let rec pat_hash bnd bv p = match p.pat_node with
     | Pwild -> bnd, bv, 0
     | Pvar v -> bnd + 1, Mvs.add v bnd bv, bnd + 1
@@ -449,7 +449,10 @@ let t_hash trigger attr t =
       | Trm (t,vml) ->
           begin match t.t_node with
           | Tvar v -> vs_hash v
-          | Tconst c -> Hashtbl.hash c
+          | Tconst c when const -> Hashtbl.hash c
+          | Tconst (Constant.ConstInt {Number.il_int = c}) -> Hashtbl.hash c
+          | Tconst (Constant.ConstReal {Number.rl_real = c}) -> Hashtbl.hash c
+          | Tconst (Constant.ConstStr c) -> Hashtbl.hash c
           | Tapp (s,l) ->
               Hashcons.combine_list (t_hash bnd vml) (ls_hash s) l
           | Tif (f,t,e) ->
@@ -498,35 +501,60 @@ let t_hash trigger attr t =
     end in
   t_hash 0 [] t
 
-let t_hash_strict t = t_hash true true t
-let t_equal_strict t1 t2 = t_compare true true true t1 t2 = 0
-let t_compare_strict t1 t2 = t_compare true true true t1 t2
+let t_hash_generic ~trigger ~attr ~const t =
+  t_hash ~trigger ~attr ~const t
+let t_compare_generic ~trigger ~attr ~loc ~const t1 t2=
+  t_compare ~trigger ~attr ~loc ~const t1 t2
+let t_equal_generic ~trigger ~attr ~loc ~const t1 t2 =
+  t_compare ~trigger ~attr ~loc ~const t1 t2 = 0
 
-module TermOHT_strict = struct
-  type t = term
-  let hash = t_hash_strict
-  let equal = t_equal_strict
-  let compare = t_compare_strict
-end
+let mterm_generic ~trigger ~attr ~loc ~const
+    : (module (Extmap.S with type key = term)) =
+  (module (Extmap.Make(struct
+      type t = term
+      let compare t1 t2 = t_compare ~trigger ~attr ~loc ~const t1 t2
+    end)))
 
-module Mterm_strict = Extmap.Make(TermOHT_strict)
-module Sterm_strict = Extset.MakeOfMap(Mterm_strict)
-module Hterm_strict = Exthtbl.Make(TermOHT_strict)
+let sterm_generic ~trigger ~attr ~loc ~const
+    : (module (Extset.S with type M.key = term)) =
+  let module M = (val (mterm_generic ~trigger ~attr ~loc ~const)) in
+  (module (Extset.MakeOfMap(M)))
 
-let t_hash t = t_hash false false t
-let t_equal t1 t2 = t_compare false false false t1 t2 = 0
-let t_compare t1 t2 = t_compare false false false t1 t2
+let hterm_generic ~trigger ~attr ~loc ~const
+    : (module (Exthtbl.S with type key = term)) =
+  (module (Exthtbl.Make(struct
+      type t = term
+      let hash t = t_hash ~trigger ~attr ~const t
+      let equal t1 t2 = t_compare ~trigger ~attr ~loc ~const t1 t2 = 0
+    end)))
 
-module TermOHT = struct
-  type t = term
-  let hash = t_hash
-  let equal = t_equal
-  let compare = t_compare
-end
+let t_hash_strict t =
+  t_hash ~trigger:true ~attr:true ~const:true t
+let t_equal_strict t1 t2 =
+  t_compare ~trigger:true ~attr:true ~loc:true ~const:true t1 t2 = 0
+let t_compare_strict t1 t2 =
+  t_compare ~trigger:true ~attr:true ~loc:true ~const:true t1 t2
 
-module Mterm = Extmap.Make(TermOHT)
-module Sterm = Extset.MakeOfMap(Mterm)
-module Hterm = Exthtbl.Make(TermOHT)
+module Mterm_strict =
+  (val (mterm_generic ~trigger:true ~attr:true ~loc:true ~const:true))
+module Sterm_strict =
+  (val (sterm_generic ~trigger:true ~attr:true ~loc:true ~const:true))
+module Hterm_strict=
+  (val (hterm_generic ~trigger:true ~attr:true ~loc:true ~const:true))
+
+let t_hash t =
+  t_hash ~trigger:false ~attr:false ~const:false t
+let t_equal t1 t2 =
+  t_compare ~trigger:false ~attr:false ~loc:false ~const:false t1 t2 = 0
+let t_compare t1 t2 =
+  t_compare ~trigger:false ~attr:false ~loc:false ~const:false t1 t2
+
+module Mterm =
+  (val (mterm_generic ~trigger:false ~attr:false ~loc:false ~const:false))
+module Sterm =
+  (val (sterm_generic ~trigger:false ~attr:false ~loc:false ~const:false))
+module Hterm =
+  (val (hterm_generic ~trigger:false ~attr:false ~loc:false ~const:false))
 
 (* type checking *)
 
@@ -797,7 +825,8 @@ let t_open_quant (vl,b,tl,f) =
   let tl = tr_map (t_subst_unsafe m) tl in
   vl, tl, t_subst_unsafe m f
 
-let t_clone_bound_id (v,_,_) = id_clone v.vs_name
+let t_clone_bound_id ?loc ?attrs (v,_,_) =
+  id_clone ?loc ?attrs v.vs_name
 
 (** open bindings with optimized closing callbacks *)
 
@@ -1596,24 +1625,46 @@ let small t = match t.t_node with
 *)
   | _ -> false
 
-let t_let_simp e ((v,b,t) as bt) =
+let v_copy_unused v =
+  let id = v.vs_name in
+  let attrs = Sattr.singleton Ident.unused_attr in
+  let attrs =
+    try
+      ignore (get_model_trace_attr ~attrs:id.id_attrs); attrs
+    with Not_found ->
+      Sattr.add (create_model_trace_attr id.id_string) attrs
+  in
+  let id' = id_derive ~attrs (id.id_string ^ unused_suffix) id in
+  create_vsymbol id' v.vs_ty
+
+let t_let_simp_keep_var ~keep e ((v,b,t) as bt) =
   let n = t_v_occurs v t in
   if n = 0 then
-    t_subst_unsafe b.bv_subst t else
+    let t = t_subst_unsafe b.bv_subst t in
+    if keep then t_let_close (v_copy_unused v) e t else t
+  else
   if n = 1 || small e then begin
     vs_check v e;
-    t_subst_unsafe (Mvs.add v e b.bv_subst) t
+    let t = t_subst_unsafe (Mvs.add v e b.bv_subst) t in
+    if keep then t_let_close (v_copy_unused v) e t else t
   end else
     t_let e bt
 
-let t_let_close_simp v e t =
+let t_let_simp = t_let_simp_keep_var ~keep:false
+
+let t_let_close_simp_keep_var ~keep v e t =
   let n = t_v_occurs v t in
-  if n = 0 then t else
+  if n = 0 then
+    if keep then t_let_close (v_copy_unused v) e t else t
+  else
   if n = 1 || small e then
-    t_subst_single v e t
+    let t = t_subst_single v e t in
+    if keep then t_let_close (v_copy_unused v) e t else t
   else
     t_let_close v e t
 
+let t_let_close_simp = t_let_close_simp_keep_var ~keep:false
+
 let t_case_simp t bl =
   let e0,tl = match bl with
     | [] -> raise EmptyCase
