@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -163,10 +163,10 @@ let meta_float = register_meta "float_type" [MTtysymbol; MTlsymbol; MTlsymbol]
     ~desc:"Projection@ and@ finiteness@ of@ a@ floating-point@ type."
 
 let meta_projection = register_meta "model_projection" [MTlsymbol]
-  ~desc:"Declares@ the@ projection."
+  ~desc:"Declare@ the@ projection."
 
 let meta_record = register_meta "model_record" [MTlsymbol]
-  ~desc:"Declares@ the@ record field."
+  ~desc:"Declare@ the@ record field."
 
 (** Theory *)
 
@@ -470,8 +470,6 @@ let add_symbol_pr uc pr = add_symbol add_pr pr.pr_name pr uc
 let create_decl d = mk_tdecl (Decl d)
 
 let print_id fmt id = Ident.print_decoded fmt id.id_string
-let print_vs fmt vs = print_id fmt vs.vs_name
-let print_tv fmt tv = Format.pp_print_char fmt '\''; print_id fmt tv.tv_name
 
 let warn_dubious_axiom uc k p syms =
   match k with
@@ -673,7 +671,7 @@ type bad_instance =
   | BadI_field_ghost of tysymbol * vsymbol (* incompatible ghost status *)
   | BadI_field_mut of tysymbol * vsymbol (* incompatible mutability *)
   | BadI_field_inv of tysymbol * vsymbol (* strengthened invariant *)
-  | BadI_ls_type of lsymbol (* lsymbol type mismatch *)
+  | BadI_ls_type of lsymbol * ty * ty (* lsymbol type mismatch *)
   | BadI_ls_kind of lsymbol (* function/predicate mismatch *)
   | BadI_ls_arity of lsymbol (* lsymbol arity mismatch *)
   | BadI_ls_rs of lsymbol (* "val function" -> "function" *)
@@ -1008,179 +1006,3 @@ let add_decl_with_tuples uc d =
   let ixs = Sid.fold add ids Sint.empty in
   let add n uc = use_export uc (tuple_theory n) in
   add_decl ~warn:false (Sint.fold add ixs uc) d
-
-(* Exception reporting *)
-
-let print_meta_arg_type fmt = function
-  | MTty -> fprintf fmt "type"
-  | MTtysymbol -> fprintf fmt "type_symbol"
-  | MTlsymbol -> fprintf fmt "logic_symbol"
-  | MTprsymbol -> fprintf fmt "proposition"
-  | MTstring -> fprintf fmt "string"
-  | MTint -> fprintf fmt "int"
-  | MTid -> fprintf fmt "identifier"
-
-let () = Exn_printer.register
-  begin fun fmt exn -> match exn with
-  | NonLocal id -> Format.fprintf fmt
-      "Non-local symbol: %a" print_id id
-  | CannotInstantiate id -> Format.fprintf fmt
-      "Cannot instantiate a defined symbol %a" print_id id
-  | BadInstance (BadI id) -> Format.fprintf fmt
-      "Illegal instantiation for symbol %a" print_id id
-  | BadInstance (BadI_ty_vars ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-          extra type variables in the type expression"
-        print_id ts.ts_name
-  | BadInstance (BadI_ty_ner ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-          record types cannot be instantiated with type expressions"
-        print_id ts.ts_name
-  | BadInstance (BadI_ty_impure ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-          both %a and the refining type expression must be pure"
-        print_id ts.ts_name print_id ts.ts_name
-  | BadInstance (BadI_ty_arity ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\narity mismatch"
-        print_id ts.ts_name
-  | BadInstance (BadI_ty_rec ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        the refining type must be a non-recursive record"
-        print_id ts.ts_name
-  | BadInstance (BadI_ty_mut_lhs ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        the refinining type must be mutable"
-        print_id ts.ts_name
-  | BadInstance (BadI_ty_mut_rhs ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        the refinining type must be immutable"
-        print_id ts.ts_name
-  | BadInstance (BadI_ty_alias ts) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        the added fields are aliased with the original fields"
-        print_id ts.ts_name
-  | BadInstance (BadI_field (ts,vs)) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        field %a not found in the refinining type"
-        print_id ts.ts_name print_id vs.vs_name
-  | BadInstance (BadI_field_type (ts,vs)) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        incompatible types for field %a"
-        print_id ts.ts_name print_id vs.vs_name
-  | BadInstance (BadI_field_ghost (ts,vs)) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        incompatible ghost status for field %a"
-        print_id ts.ts_name print_id vs.vs_name
-  | BadInstance (BadI_field_mut (ts,vs)) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        incompatible mutability status for field %a"
-        print_id ts.ts_name print_id vs.vs_name
-  | BadInstance (BadI_field_inv (ts,vs)) -> Format.fprintf fmt
-      "Illegal instantiation for type %a:@\n\
-        field %a must not appear in the refined invariant"
-        print_id ts.ts_name print_id vs.vs_name
-  | BadInstance (BadI_ls_type ls) -> Format.fprintf fmt
-      "Illegal instantiation for %s %a:@\ntype mismatch"
-        (if ls.ls_value = None then "predicate" else "function")
-        print_id ls.ls_name
-  | BadInstance (BadI_ls_kind ls) -> Format.fprintf fmt
-      "Illegal instantiation for %s %a:@\n%s expected"
-        (if ls.ls_value = None then "predicate" else "function")
-        print_id ls.ls_name
-        (if ls.ls_value = None then "predicate" else "function")
-  | BadInstance (BadI_ls_arity ls) -> Format.fprintf fmt
-      "Illegal instantiation for %s %a:@\narity mismatch"
-        (if ls.ls_value = None then "predicate" else "function")
-        print_id ls.ls_name
-  | BadInstance (BadI_ls_rs ls) -> Format.fprintf fmt
-      "Cannot instantiate %s %a:@\nprogram function %a \
-        must be refined instead"
-        (if ls.ls_value = None then "predicate" else "function")
-        print_id ls.ls_name print_id ls.ls_name
-  | BadInstance (BadI_rs_arity id) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        arity mismatch"
-        print_id id
-  | BadInstance (BadI_rs_type (id,exn)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        %a"
-        print_id id Exn_printer.exn_printer exn
-  | BadInstance (BadI_rs_kind id) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        incompatible kind"
-        print_id id
-  | BadInstance (BadI_rs_ghost id) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        incompatible ghost status"
-        print_id id
-  | BadInstance (BadI_rs_mask id) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        incompatible mask"
-        print_id id
-  | BadInstance (BadI_rs_reads (id,svs)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        unreferenced external dependencies: %a"
-        print_id id (Pp.print_list Pp.space print_vs) (Svs.elements svs)
-  | BadInstance (BadI_rs_writes (id,svs)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        unreferenced write effects in variables: %a"
-        print_id id (Pp.print_list Pp.space print_vs) (Svs.elements svs)
-  | BadInstance (BadI_rs_taints (id,svs)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        unreferenced ghost write effects in variables: %a"
-        print_id id (Pp.print_list Pp.space print_vs) (Svs.elements svs)
-  | BadInstance (BadI_rs_covers (id,svs)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        unreferenced region modifications in variables: %a"
-        print_id id (Pp.print_list Pp.space print_vs) (Svs.elements svs)
-  | BadInstance (BadI_rs_resets (id,svs)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        unreferenced region resets in variables: %a"
-        print_id id (Pp.print_list Pp.space print_vs) (Svs.elements svs)
-  | BadInstance (BadI_rs_raises (id,sid)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        unreferenced raised exceptions: %a"
-        print_id id (Pp.print_list Pp.space print_id) (Sid.elements sid)
-  | BadInstance (BadI_rs_spoils (id,stv)) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        restricted type variables: %a"
-        print_id id (Pp.print_list Pp.space print_tv) (Stv.elements stv)
-  | BadInstance (BadI_rs_oneway id) -> Format.fprintf fmt
-      "Illegal instantiation for program function %a:@\n\
-        incompatible termination status"
-        print_id id
-  | BadInstance (BadI_xs_type id) -> Format.fprintf fmt
-      "Illegal instantiation for exception %a:@\n\
-        type mismatch"
-        print_id id
-  | BadInstance (BadI_xs_mask id) -> Format.fprintf fmt
-      "Illegal instantiation for exception %a:@\n\
-        incompatible mask"
-        print_id id
-  | CloseTheory ->
-      Format.fprintf fmt "Cannot close theory: some namespaces are still open"
-  | NoOpenedNamespace ->
-      Format.fprintf fmt "No opened namespaces"
-  | ClashSymbol s ->
-      Format.fprintf fmt "Symbol %s is already defined in the current scope" s
-  | UnknownMeta s ->
-      Format.fprintf fmt "Unknown meta-property %s" s
-  | KnownMeta m ->
-      Format.fprintf fmt "Metaproperty %s is already registered with \
-        a conflicting signature" m.meta_name
-  | BadMetaArity (m,n) ->
-      let i = List.length m.meta_type in
-      Format.fprintf fmt "Metaproperty %s expects %d argument%s but \
-        is applied to %d" m.meta_name i (if i = 1 then "" else "s") n
-  | MetaTypeMismatch (m,t1,t2) ->
-      Format.fprintf fmt "Metaproperty %s expects a %a argument but \
-        is applied to %a"
-        m.meta_name print_meta_arg_type t1 print_meta_arg_type t2
-  | RangeConflict ts ->
-      Format.fprintf fmt "Conflicting definitions for range type %s"
-        ts.ts_name.id_string
-  | FloatConflict ts ->
-      Format.fprintf fmt "Conflicting definitions for float type %s"
-        ts.ts_name.id_string
-  | _ -> raise exn
-  end
