@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,10 +9,10 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Wstdlib
-
 (** Theories and Namespaces *)
 
+open Wstdlib
+
 open Ident
 open Ty
 open Term
@@ -251,12 +251,12 @@ type bad_instance =
   | BadI_field_ghost of tysymbol * vsymbol (* incompatible ghost status *)
   | BadI_field_mut of tysymbol * vsymbol (* incompatible mutability *)
   | BadI_field_inv of tysymbol * vsymbol (* strengthened invariant *)
-  | BadI_ls_type of lsymbol (* lsymbol type mismatch *)
+  | BadI_ls_type of lsymbol * ty * ty (* lsymbol type mismatch *)
   | BadI_ls_kind of lsymbol (* function/predicate mismatch *)
   | BadI_ls_arity of lsymbol (* lsymbol arity mismatch *)
   | BadI_ls_rs of lsymbol (* "val function" -> "function" *)
   | BadI_rs_arity of ident (* incompatible rsymbol arity *)
-  | BadI_rs_type of ident (* rsymbol type mismatch *)
+  | BadI_rs_type of ident * exn (* rsymbol type mismatch *)
   | BadI_rs_kind of ident (* incompatible rsymbol kind *)
   | BadI_rs_ghost of ident (* incompatible ghost status *)
   | BadI_rs_mask of ident (* incompatible result mask *)
