@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -25,6 +25,14 @@ let driver_debug =
   Debug.register_flag "interm_task"
     ~desc:"Print intermediate task generated during processing of a driver"
 
+let meta_get_counterexmp =
+  Theory.register_meta_excl "get_counterexmp" [Theory.MTstring]
+  ~desc:"Set@ when@ counter-example@ should@ be@ get."
+
+let get_counterexmp task =
+  let ce_meta = Task.find_meta_tds task meta_get_counterexmp in
+  not (Theory.Stdecl.is_empty ce_meta.tds_set)
+
 (** drivers *)
 
 type driver = {
@@ -251,7 +259,7 @@ exception UnknownSpec of string
 
 let filename_regexp = Re.Str.regexp "%\\(.\\)"
 
-let get_filename drv input_file theory_name goal_name =
+let get_filename drv ~input_file ~theory_name ~goal_name =
   let sanitize = Ident.sanitizer
     Ident.char_to_alnumus Ident.char_to_alnumus in
   let file = match drv.drv_filename with
@@ -268,16 +276,18 @@ let get_filename drv input_file theory_name goal_name =
   Re.Str.global_substitute filename_regexp replace file
 
 let file_of_task drv input_file theory_name task =
-  get_filename drv input_file theory_name (task_goal task).pr_name.id_string
+  get_filename drv ~input_file ~theory_name ~goal_name:(task_goal task).pr_name.id_string
 
 let file_of_theory drv input_file th =
-  get_filename drv input_file th.th_name.Ident.id_string "null"
+  get_filename drv ~input_file ~theory_name:th.th_name.Ident.id_string ~goal_name:"null"
 
-let call_on_buffer ~command ~limit ~gen_new_file ?inplace ~filename
-    ~printer_mapping drv buffer =
+let call_on_buffer
+      ~command ~libdir ~datadir ~limit ~gen_new_file ?inplace ~filename
+    ~printing_info drv buffer =
   Call_provers.call_on_buffer
-    ~command ~limit ~gen_new_file ~res_parser:drv.drv_res_parser
-    ~filename ~printer_mapping ?inplace buffer
+    ~command ~libdir ~datadir ~limit ~gen_new_file
+    ~res_parser:drv.drv_res_parser
+    ~filename ~printing_info ?inplace buffer
 
 (** print'n'prove *)
 
@@ -346,15 +356,15 @@ let print_task_prepared ?old drv fmt task =
     | None -> raise NoPrinter
     | Some p -> p
   in
-  let printer_args = { Printer.env = drv.drv_env;
-      prelude     = drv.drv_prelude;
-      th_prelude  = drv.drv_thprelude;
-      blacklist   = drv.drv_blacklist;
-      printer_mapping = get_default_printer_mapping;
+  let printer_args = Printer.{
+      env           = drv.drv_env;
+      prelude       = drv.drv_prelude;
+      th_prelude    = drv.drv_thprelude;
+      blacklist     = drv.drv_blacklist;
+      printing_info = ref None;
     } in
-  let printer = lookup_printer p printer_args in
-  fprintf fmt "@[%a@]@?" (printer ?old) task;
-  printer_args.printer_mapping
+  fprintf fmt "@[%a@]@?" (lookup_printer p ?old printer_args) task;
+  Opt.get_def default_printing_info !(printer_args.printing_info)
 
 let print_task ?old drv fmt task =
   let task = prepare_task drv task in
@@ -388,7 +398,10 @@ let file_name_of_task ?old ?inplace ?interactive drv task =
           | Some loc -> let fn,_,_,_ = Loc.get loc in Filename.basename fn
           | None -> "" in
         let fn = try Filename.chop_extension fn with Invalid_argument _ -> fn in
-        true, get_filename drv fn "T" pr.pr_name.id_string
+        true, get_filename drv
+          ~input_file:fn
+          ~theory_name:"T"
+          ~goal_name:pr.pr_name.id_string
     | _ ->
         (* Example: cvc4 without ?save_to argument
            No file were provided. We have to generate a new one.
@@ -398,26 +411,35 @@ let file_name_of_task ?old ?inplace ?interactive drv task =
           | Some loc -> let fn,_,_,_ = Loc.get loc in Filename.basename fn
           | None -> "" in
         let fn = try Filename.chop_extension fn with Invalid_argument _ -> fn in
-        true, get_filename drv fn "T" pr.pr_name.id_string
+        true, get_filename drv
+          ~input_file:fn
+          ~theory_name:"T"
+          ~goal_name:pr.pr_name.id_string
 
-let prove_task_prepared ~command ~limit ?old ?inplace ?interactive drv task =
+let prove_task_prepared
+      ~command ~libdir ~datadir ~limit ?old ?inplace ?interactive drv task =
   let buf = Buffer.create 1024 in
   let fmt = formatter_of_buffer buf in
-  let gen_new_file, filename =
-    file_name_of_task ?old ?inplace ?interactive drv task in
   let old_channel = Opt.map open_in old in
-  let printer_mapping = print_task_prepared ?old:old_channel drv fmt task in
+  let printing_info = print_task_prepared ?old:old_channel drv fmt task in
   pp_print_flush fmt ();
   Opt.iter close_in old_channel;
+  let gen_new_file, filename =
+    file_name_of_task ?old ?inplace ?interactive drv task in
+  let get_counterexmp = get_counterexmp task in
   let res =
-    call_on_buffer ~command ~limit ~gen_new_file ?inplace
-      ~filename ~printer_mapping drv buf in
+    call_on_buffer
+      ~command ~libdir ~datadir ~limit ~gen_new_file ?inplace ~filename
+      ~get_counterexmp ~printing_info drv buf
+  in
   Buffer.reset buf;
   res
 
-let prove_task ~command ~limit ?old ?inplace ?interactive drv task =
+let prove_task
+      ~command ~libdir ~datadir ~limit ?old ?inplace ?interactive drv task =
   let task = prepare_task drv task in
-  prove_task_prepared ~command ~limit ?interactive ?old ?inplace drv task
+  prove_task_prepared
+    ~command ~libdir ~datadir ~limit ?interactive ?old ?inplace drv task
 
 (* exception report *)
 
