@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -113,7 +113,7 @@ let connect_external socket_name =
   Buffer.clear recv_buf;
   client_connect ~fail:true socket_name
 
-let connect_internal () =
+let connect_internal libdir =
   if is_connected () then raise AlreadyConnected;
   Buffer.clear recv_buf;
   let cwd = Unix.getcwd () in
@@ -122,7 +122,7 @@ let connect_internal () =
     ("why3server." ^ string_of_int (Unix.getpid ()) ^ ".sock") *)
     Filename.temp_file "why3server" "sock"
   in
-  let exec = Filename.concat Config.libdir "why3server" in
+  let exec = Filename.concat libdir "why3server" in
   let pid =
     (* use this version for debugging the C code
     Unix.create_process "valgrind"
@@ -154,8 +154,8 @@ let connect_internal () =
 let disconnect = client_disconnect
 
 (* TODO/FIXME: is this the right place to connect-on-demand? *)
-let send_request ~id ~timelimit ~memlimit ~use_stdin ~cmd =
-  if not (is_connected ()) then connect_internal ();
+let send_request ~libdir ~id ~timelimit ~memlimit ~use_stdin ~cmd =
+  if not (is_connected ()) then connect_internal libdir;
   Buffer.clear send_buf;
   let servercommand =
     if use_stdin <> None then "runstdin;" else "run;" in
@@ -179,8 +179,8 @@ let send_request ~id ~timelimit ~memlimit ~use_stdin ~cmd =
   let s = Buffer.contents send_buf in
   send_request_string s
 
-let send_interrupt ~id =
-  if not (is_connected ()) then connect_internal ();
+let send_interrupt ~libdir ~id =
+  if not (is_connected ()) then connect_internal libdir;
   Buffer.clear send_buf;
   Buffer.add_string send_buf "interrupt;";
   Buffer.add_string send_buf (string_of_int id);
@@ -247,9 +247,9 @@ let read_answers ~blocking =
 
 let () = Exn_printer.register (fun fmt exn -> match exn with
   | NotConnected ->
-      Format.fprintf fmt "Not connected to the proof server"
+      Format.pp_print_string fmt "Not connected to the proof server"
   | AlreadyConnected ->
-      Format.fprintf fmt "Already connected to the proof server"
+      Format.pp_print_string fmt "Already connected to the proof server"
   | InvalidAnswer s ->
       Format.fprintf fmt "Invalid server answer: %s" s
   | ConnectionError s ->
