@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -27,6 +27,9 @@ type sexp =
   | Atom of string
   | List of sexp list
 
+val exists : (sexp -> bool) -> sexp -> bool
+(** Check if the node or any of its sub-nodes satisfies the predicate. *)
+
 exception Error
 
 val read : Lexing.lexbuf -> sexp
