@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,7 +9,6 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Compile
 open Format
 open Ident
 open Pp
@@ -42,7 +41,7 @@ module Print = struct
   end)
 
   let print_vsty info fmt (id, _, _) =
-    fprintf fmt "%a" (print_lident info) id
+    print_lident info fmt id
 
   let print_tv_arg = print_tv
   let print_tv_args fmt = function
@@ -64,7 +63,7 @@ module Print = struct
 
   let rec print_pat info fmt = function
     | Pwild ->
-        fprintf fmt "_"
+        pp_print_string fmt "_"
     | Pvar {vs_name = id} ->
         print_lident info fmt id
     | Pas (p, {vs_name=id}) ->
@@ -86,7 +85,7 @@ module Print = struct
                 (pjl, pl)
 
   and print_papp info ls fmt = function
-    | []  -> fprintf fmt "%a"      (print_uident info) ls.ls_name
+    | []  -> print_uident info fmt ls.ls_name
     | [p] -> fprintf fmt "%a %a"   (print_uident info) ls.ls_name
                (print_pat info) p
     | pl  -> fprintf fmt "%a %a" (print_uident info) ls.ls_name
@@ -97,15 +96,6 @@ module Print = struct
   let pv_name pv = pv.pv_vs.vs_name
   let print_pv info fmt pv = print_lident info fmt (pv_name pv)
 
-  (* FIXME put these in Compile*)
-  let is_true e = match e.e_node with
-    | Eapp (s, []) -> rs_equal s rs_true
-    | _ -> false
-
-  let is_false e = match e.e_node with
-    | Eapp (s, []) -> rs_equal s rs_false
-    | _ -> false
-
   let is_mapped_to_int info ity =
     match ity.ity_node with
     | Ityapp ({ its_ts = ts }, _, _) ->
@@ -126,22 +116,22 @@ module Print = struct
         let v = c.Number.il_int in
         let s = BigInt.to_string v in
         if BigInt.lt v BigInt.zero then fprintf fmt "(%s)" s
-        else fprintf fmt "%s" s
+        else pp_print_string fmt s
     | Econst (Constant.ConstStr s) ->
        Constant.print_string_constant escape fmt s
     | _ -> assert false end
 
   let print_for_direction fmt = function
-    | To     -> fprintf fmt "to"
-    | DownTo -> fprintf fmt "downto"
+    | To     -> pp_print_string fmt "to"
+    | DownTo -> pp_print_string fmt "downto"
 
   let rec print_apply_args info fmt = function
     | expr :: exprl, _ :: pvl ->
-        fprintf fmt "%a" (print_expr ~paren:true info) expr;
+        print_expr ~paren:true info fmt expr;
         if exprl <> [] then fprintf fmt "@ ";
         print_apply_args info fmt (exprl, pvl)
     | expr :: exprl, [] ->
-        fprintf fmt "%a" (print_expr ~paren:true info) expr;
+        print_expr ~paren:true info fmt expr;
         print_apply_args info fmt (exprl, [])
     | [], _ -> ()
 
@@ -176,7 +166,7 @@ module Print = struct
               fprintf fmt "@[<hov 2>%a %a@]" (print_uident info) rs.rs_name
                 (print_list space (print_expr ~paren:true info)) tl
           | pjl, tl ->
-              let equal fmt () = fprintf fmt " = " in
+              let equal fmt () = pp_print_string fmt " = " in
               fprintf fmt "@[<hov 2>{ @[%a@] }@]"
                 (print_list2 semi equal (print_rs info) (print_expr info))
                 (pjl, tl) end
@@ -186,9 +176,6 @@ module Print = struct
         fprintf fmt "@[<hov 2>%a %a@]" (print_lident info) rs.rs_name
           (print_apply_args info) (tl, rs.rs_cty.cty_args)
 
-  and print_svar fmt s =
-    Stv.iter (fun tv -> fprintf fmt "%a " print_tv tv) s
-
   and print_fun_type_args info fmt (args, _, _, e) =
     (* TODO: search if CakeML supports some form of polymorphic recursion *)
     (* if Stv.is_empty s then *)
@@ -238,8 +225,8 @@ module Print = struct
     | Econst (Constant.ConstInt c) ->
         let n = c.Number.il_int in
         let n = BigInt.to_string n in
-        let id = match e.e_ity with
-          | I { ity_node = Ityapp ({its_ts = ts},_,_) } -> ts.ts_name
+        let id = match e.e_mlty with
+          | Tapp (tname, _) -> tname
           | _ -> assert false in
         (match query_syntax info.info_literal id with
          | Some s -> syntax_arguments s print_constant fmt [e]
@@ -255,13 +242,11 @@ module Print = struct
         forget_let_defn let_def
     | Eabsurd ->
         fprintf fmt (protect_on paren "assert false (* absurd *)")
-    | Eapp (rs, []) when rs_equal rs rs_true ->
-        fprintf fmt "true"
-    | Eapp (rs, []) when rs_equal rs rs_false ->
-        fprintf fmt "false"
-    | Eapp (rs, [])  -> (* avoids parenthesis around values *)
-        fprintf fmt "%a" (print_apply info rs) []
-    | Eapp (rs, pvl) ->
+    | Eapp (rs, [], false) when rs_equal rs rs_true ->
+        pp_print_string fmt "true"
+    | Eapp (rs, [], false) when rs_equal rs rs_false ->
+        pp_print_string fmt "false"
+    | Eapp (rs, pvl, _) ->
        fprintf fmt (protect_on paren "%a")
                (print_apply info rs) pvl
     | Ematch (e1, [p, e2], []) ->
@@ -281,10 +266,10 @@ module Print = struct
           (print_expr info) e (print_list_next newline (print_branch info)) bl
           (print_list_next newline (print_xbranch true info)) xl
     | Eassign al ->
-        let assign fmt (rho, rs, e) =
+        let assign fmt (e1, _, rs, e2) =
           fprintf fmt "@[<hov 2>%a.%a <-@ %a@]"
-            (print_lident info) (pv_name rho) (print_lident info) rs.rs_name
-            (print_expr info) e in
+            (print_expr info) e1 (print_lident info) rs.rs_name
+            (print_expr info) e2 in
         begin match al with
           | [] -> assert false | [a] -> assign fmt a
           | al -> fprintf fmt "@[(%a)]" (print_list semi assign) al end
@@ -307,7 +292,7 @@ module Print = struct
              "@[<hv>@[<hov 2>if@ %a@ then@ (@[%a@])@] @;<1 0>else (@[%a@])@]")
           (print_expr info) e1 (print_expr info) e2 (print_expr info) e3
     | Eblock [] ->
-        fprintf fmt "()"
+        pp_print_string fmt "()"
     | Eblock [e] ->
         print_expr info fmt e
     | Eblock el ->
@@ -320,7 +305,7 @@ module Print = struct
           (print_expr info) e1 (print_expr info) e2
     | Eraise (xs, e_opt) ->
         print_raise ~paren info xs fmt e_opt
-    | Efor (pv1, pv2, dir, pv3, e) ->
+    | Efor (pv1, _ty, pv2, dir, pv3, e) ->
         if is_mapped_to_int info pv1.pv_ity then
           fprintf fmt "@[<hov 2>for %a = %a %a %a do@ @[%a@]@ done@]"
             (print_lident info) (pv_name pv1) (print_lident info) (pv_name pv2)
@@ -369,7 +354,7 @@ module Print = struct
 
   and print_xbranch case info fst fmt (xs, pvl, e) =
     let print_exn fmt () =
-      if case then fprintf fmt "exception " else fprintf fmt "" in
+      if case then pp_print_string fmt "exception " in
     let print_var fmt pv = print_lident info fmt (pv_name pv) in
     match query_syntax info.info_syn xs.xs_name, pvl with
     | Some s, _ ->
@@ -485,8 +470,15 @@ let cml_printer = Pdriver.{
     desc = "printer for CakeML code";
     implem_printer = {
         filename_generator = fg_cml;
-        decl_printer = print_decl;
-        prelude_printer = dummy_prelude_printer;
+        decl_printer = print_decl ~flat:false;
+        prelude_printer = default_prelude_printer;
+        header_printer = dummy_border_printer;
+        footer_printer = dummy_border_printer;
+      };
+    flat_printer = {
+        filename_generator = fg_cml;
+        decl_printer = print_decl ~flat:true;
+        prelude_printer = default_prelude_printer;
         header_printer = dummy_border_printer;
         footer_printer = dummy_border_printer;
       };
@@ -504,8 +496,15 @@ let sml_printer = Pdriver.{
     desc = "printer for SML code";
     implem_printer = {
         filename_generator = fg_sml;
-        decl_printer = print_decl;
-        prelude_printer = dummy_prelude_printer;
+        decl_printer = print_decl ~flat:false;
+        prelude_printer = default_prelude_printer;
+        header_printer = dummy_border_printer;
+        footer_printer = dummy_border_printer;
+      };
+    flat_printer = {
+        filename_generator = fg_sml;
+        decl_printer = print_decl ~flat:true;
+        prelude_printer = default_prelude_printer;
         header_printer = dummy_border_printer;
         footer_printer = dummy_border_printer;
       };
