@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -39,8 +39,6 @@ type value =
 and field = Fimmutable of value | Fmutable of value ref
 
 type info = {
-    env : Env.env;
-    mm  : Pmodule.pmodule Mstr.t;
     vars: value Mid.t;
     funs: decl Mrs.t;
     get_decl: rsymbol -> Mltree.decl;
@@ -75,11 +73,11 @@ let value_of_const c ty = match c with
 open Format
 
 let rec print_value fmt = function
-  | Vvoid -> fprintf fmt "()"
+  | Vvoid -> pp_print_string fmt "()"
   | Vbool b -> fprintf fmt "%b" b
   | Vstring s -> Constant.(print_string_def fmt s)
   | Vbigint i -> Constant.print_def fmt (Constant.int_const i)
-  | Vint i -> fprintf fmt "%d" i
+  | Vint i -> pp_print_int fmt i
   | Vtuple l -> fprintf fmt "@[<hov 2>(%a)@]"
                         (Pp.print_list Pp.comma print_value) l
   | Vconstr (rs, lf) -> fprintf fmt "@[<hov 2>(%a@ %a)@]"
@@ -91,7 +89,7 @@ let rec print_value fmt = function
   | Vref r -> fprintf fmt "Vref %a" print_value !r
 
 and print_field fmt = function
-  | Fimmutable v -> fprintf fmt "%a" print_value v
+  | Fimmutable v -> print_value fmt v
   | Fmutable vr -> fprintf fmt "Fmutable %a" print_value !vr
 and print_matrix fmt m =
   Array.iter (fun a -> fprintf fmt "[|%a|]\n"
@@ -544,7 +542,7 @@ let ts = ref 0. (* timestamp for current callstack *)
 
 let print_callstack cs time =
   Format.eprintf "%a %d@."
-    (Pp.print_list (fun fmt () -> Format.fprintf fmt ";") Expr.print_rs)
+    (Pp.print_list (fun fmt () -> Format.pp_print_string fmt ";") Expr.print_rs)
     (List.rev cs)
     (int_of_float (time *. 100000000.))
 
@@ -615,7 +613,7 @@ let rec interp_expr info (e:Mltree.expr) : value =
       with Not_found ->
         Debug.dprintf debug_interp "var %a not found@." print_pv pv;
            raise CannotReduce)
-  | Eapp (rs, le) -> begin
+  | Eapp (rs, le, _) -> begin
       Debug.dprintf debug_interp "Eapp %a@." Expr.print_rs rs;
       let eval_call info vl e rs =
         Debug.dprintf debug_interp "eval params@.";
@@ -702,9 +700,12 @@ let rec interp_expr info (e:Mltree.expr) : value =
      end
   | Eassign l ->
      List.iter
-       (fun (pvs, rs, e) ->
+       (fun (epv, _, rs, e) ->
          let fld = fd_of_rs rs in
          let value = interp_expr info e in
+         let pvs = match epv.e_node with
+           | Evar pv -> pv
+           | _ -> assert false in
          match get pvs info with
          | Vconstr(c, args) ->
             let rec aux cargs args =
@@ -735,7 +736,7 @@ let rec interp_expr info (e:Mltree.expr) : value =
         interp_expr info e
      | Vbool false -> Vvoid
      | _ -> assert false end
-  | Efor (x, pv1, dir, pv2, e) ->
+  | Efor (x, _, pv1, dir, pv2, e) ->
      Debug.dprintf debug_interp "for@.";
      begin match (get pv1 info, get pv2 info) with
      | (Vbigint i1, Vbigint i2) ->
@@ -862,9 +863,7 @@ let rec term_of_value = function
   | Vmatrix _ -> raise CannotReduce
 
 let init_info env mm rs vars =
-  { env = env;
-    mm = mm;
-    funs = Mrs.empty;
+  { funs = Mrs.empty;
     vars = vars;
     get_decl = get_decl env mm;
     cur_rs = rs;
