@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -11,7 +11,6 @@
 
 (** Printer for extracted OCaml code *)
 
-open Compile
 open Format
 open Ident
 open Pp
@@ -30,7 +29,6 @@ type info = {
   info_literal      : syntax_map;
   info_current_th   : Theory.theory;
   info_current_mo   : Pmodule.pmodule option;
-  info_th_known_map : Decl.known_map;
   info_mo_known_map : Pdecl.known_map;
   info_fname        : string option;
   info_flat         : bool;
@@ -136,7 +134,7 @@ module Print = struct
   let print_global_ident ~sanitizer fmt id =
     let s = id_unique ~sanitizer tprinter id in
     Ident.forget_id tprinter id;
-    fprintf fmt "%s" s
+    pp_print_string fmt s
 
   let print_path ~sanitizer fmt (q, id) =
     assert (List.length q >= 1);
@@ -165,12 +163,12 @@ module Print = struct
       let p, t, q =
         try Pmodule.restore_path id with Not_found -> Theory.restore_path id in
       let fname = if p = [] then info.info_fname else None in
-      let m = Strings.capitalize (module_name ?fname p t) in
+      let m = Strings.capitalize (Ml_printer.module_name ?fname p t) in
       fprintf fmt "%s.%a" m (print_path ~sanitizer) (q, id)
     with
     | Not_found ->
         let s = id_unique ~sanitizer iprinter id in
-        fprintf fmt "%s" s
+        pp_print_string fmt s
     | Local ->
         let _, _, q =
           try Pmodule.restore_path id with Not_found ->
@@ -210,10 +208,10 @@ module Print = struct
   let print_record_proj info fmt rs =
     match query_syntax info.info_syn rs.rs_name with
     | None ->
-       fprintf fmt "%a" (print_lident info) rs.rs_name
+        print_lident info fmt rs.rs_name
     | Some s when complex_syntax s ->
        Loc.errorm ?loc:rs.rs_name.id_loc "Unsupported: complex record field"
-    | Some s -> fprintf fmt "%s" s
+    | Some s -> pp_print_string fmt s
 
   (** Types *)
 
@@ -221,12 +219,16 @@ module Print = struct
     | Tvar tv ->
         print_tv ~use_quote fmt tv
     | Ttuple [] ->
-        fprintf fmt "unit"
+        pp_print_string fmt "unit"
     | Ttuple [t] ->
         print_ty  ~use_quote ~paren info fmt t
     | Ttuple tl ->
         fprintf fmt (protect_on paren "@[%a@]")
           (print_list star (print_ty ~use_quote ~paren:true info)) tl
+    | Tarrow (t1, t2) ->
+        fprintf fmt (protect_on paren "%a -> %a")
+          (print_ty ~use_quote ~paren info) t1
+          (print_ty ~use_quote ~paren info) t2
     | Tapp (ts, tl) ->
         match query_syntax info.info_syn ts with
         | Some s when complex_syntax s ->
@@ -270,12 +272,14 @@ module Print = struct
     else fprintf fmt "(%a:@ %a)" (print_lident info) id
       (print_ty ~use_quote:false ~paren:false info) ty
 
-  let print_vsty_opt_fun info fmt id = function
+  let print_vsty_opt_fun info fmt id ty = match ty with
     | Tapp (id_ty, [arg]) ->
         assert (query_syntax info.info_syn id_ty = Some "%1 option");
         fprintf fmt "?%s:%a" id.id_string
           (print_ty ~use_quote:false ~paren:false info) arg
-    | _ -> invalid_arg "print_vsty_opt_fun" (*FIXME : better error message *)
+    | _ ->
+       Loc.errorm "invalid optional argument of type %a"
+         (print_ty ~use_quote:false ~paren:false info) ty
 
   let print_vsty_named_fun info fmt id ty =
     fprintf fmt "%s:%a" id.id_string
@@ -285,7 +289,7 @@ module Print = struct
     let attrs = id.id_attrs in
     if is_optional ~attrs then print_vsty_opt_fun info fmt id ty
     else if is_named ~attrs then print_vsty_named_fun info fmt id ty
-    else fprintf fmt "%a" (print_ty ~use_quote:false ~paren:true info) ty
+    else print_ty ~use_quote:false ~paren:true info fmt ty
 
   let print_vs_fun info fmt id =
     let attrs = id.id_attrs in
@@ -314,7 +318,7 @@ module Print = struct
 
   let rec print_pat info prec fmt = function
     | Pwild ->
-        fprintf fmt "_"
+        pp_print_string fmt "_"
     | Pvar {vs_name=id} ->
         (print_lident info) fmt id
     | Pas (p, {vs_name=id}) ->
@@ -344,7 +348,7 @@ module Print = struct
                   (print_pat info 6)) (pjl, pl)
 
   and print_papp info ls fmt = function
-    | []  -> fprintf fmt "%a"      (print_uident info) ls.ls_name
+    | []  -> print_uident info fmt ls.ls_name
     | [p] -> fprintf fmt "%a %a"   (print_uident info) ls.ls_name
                (print_pat info 3) p
     | pl  -> fprintf fmt "%a (%a)" (print_uident info) ls.ls_name
@@ -355,15 +359,6 @@ module Print = struct
   let pv_name pv = pv.pv_vs.vs_name
   let print_pv info fmt pv = print_lident info fmt (pv_name pv)
 
-  (* FIXME put these in Compile*)
-  let is_true e = match e.e_node with
-    | Eapp (s, []) -> rs_equal s rs_true
-    | _ -> false
-
-  let is_false e = match e.e_node with
-    | Eapp (s, []) -> rs_equal s rs_false
-    | _ -> false
-
   let is_field rs =
       match rs.rs_field with
       | None   -> false
@@ -387,31 +382,31 @@ module Print = struct
         let v = c.Number.il_int in
         let s = BigInt.to_string v in
         if BigInt.lt v BigInt.zero then fprintf fmt "(%s)" s
-        else fprintf fmt "%s" s
+        else pp_print_string fmt s
     | Econst (Constant.ConstStr s) ->
        Constant.print_string_def fmt s
     | _ -> assert false end
 
   let print_for_direction fmt = function
-    | To     -> fprintf fmt "to"
-    | DownTo -> fprintf fmt "downto"
+    | To     -> pp_print_string fmt "to"
+    | DownTo -> pp_print_string fmt "downto"
 
   let rec print_apply_args info fmt = function
     | expr :: exprl, pv :: pvl ->
         if is_optional ~attrs:(pv_name pv).id_attrs then
           begin match expr.e_node with
-            | Eapp (rs, _)
+            | Eapp (rs, _, false)
               when query_syntax info.info_syn rs.rs_name = Some "None" -> ()
             | _ -> fprintf fmt "?%s:%a" (pv_name pv).id_string
                      (print_expr info 1) expr end
         else if is_named ~attrs:(pv_name pv).id_attrs then
           fprintf fmt "~%s:%a" (pv_name pv).id_string
             (print_expr info 1) expr
-        else fprintf fmt "%a" (print_expr info 3) expr;
+        else print_expr info 3 fmt expr;
         if exprl <> [] then fprintf fmt "@ ";
         print_apply_args info fmt (exprl, pvl)
     | expr :: exprl, [] ->
-        fprintf fmt "%a" (print_expr info 3) expr;
+        print_expr info 3 fmt expr;
         print_apply_args info fmt (exprl, [])
     | [], _ -> ()
 
@@ -543,13 +538,13 @@ module Print = struct
     | Econst (Constant.ConstInt c) ->
         let n = c.Number.il_int in
         let n = BigInt.to_string n in
-        let id = match e.e_ity with
-          | I { ity_node = Ityapp ({its_ts = ts},_,_) } -> ts.ts_name
+        let id = match e.e_mlty with
+          | Tapp (tname, _) -> tname
           | _ -> assert false in
         (match query_syntax info.info_literal id with
          | Some s -> syntax_arguments s print_constant fmt [e]
-         | None when n = "0" -> fprintf fmt "Z.zero"
-         | None when n = "1" -> fprintf fmt "Z.one"
+         | None when n = "0" -> pp_print_string fmt "Z.zero"
+         | None when n = "1" -> pp_print_string fmt "Z.one"
          | None   -> fprintf fmt (protect_on (prec < 4) "Z.of_string \"%s\"") n)
     | Econst (Constant.ConstStr s) ->
         Constant.print_string_def fmt s
@@ -562,15 +557,14 @@ module Print = struct
         forget_let_defn let_def
     | Eabsurd ->
         fprintf fmt (protect_on (opr && prec < 4) "assert false (* absurd *)")
-    | Eapp (rs, []) when rs_equal rs rs_true ->
-        fprintf fmt "true"
-    | Eapp (rs, []) when rs_equal rs rs_false ->
-        fprintf fmt "false"
-    | Eapp (rs, [])  -> (* avoids parenthesis around values *)
-        print_apply info prec rs fmt []
-    | Eapp (rs, [e]) when query_syntax info.info_syn rs.rs_name = Some "%1" ->
+    | Eapp (rs, [], _) when rs_equal rs rs_true ->
+        pp_print_string fmt "true"
+    | Eapp (rs, [], _) when rs_equal rs rs_false ->
+        pp_print_string fmt "false"
+    | Eapp (rs, [e], _)
+         when query_syntax info.info_syn rs.rs_name = Some "%1" ->
         print_expr ~boxed ~opr ~be info prec fmt e
-    | Eapp (rs, pvl) ->
+    | Eapp (rs, pvl, _) ->
         print_apply info prec rs fmt pvl
     | Ematch (e1, [p, e2], []) ->
         fprintf fmt (protect_on (opr && prec < 18) "let %a =@ %a in@ %a")
@@ -584,24 +578,25 @@ module Print = struct
           (print_expr info 18) e
           (print_list newline (print_branch info)) pl
     | Eassign al ->
-        let assign fmt (rho, rs, e) =
+        let assign fmt (e1, _, rs, e2) =
           if rs_equal rs rs_ref_proj
           then fprintf fmt "@[<hv 2>%a :=@ %a@]"
-                 (print_lident info) (pv_name rho) (print_expr info 15) e
+                 (print_expr info 14) e1 (print_expr info 15) e2
           else if is_field rs
           then fprintf fmt "@[<hv 2>%a.%a <-@ %a@]"
-                 (print_lident info) (pv_name rho) (print_record_proj info) rs
-                (print_expr info 15) e
+                 (print_expr info 14) e1 (print_record_proj info) rs
+                (print_expr info 15) e2
           else
             match query_syntax info.info_syn rs.rs_name with
             | Some s ->
                fprintf fmt "@[<hv 2>%a <-@ %a@]"
-                (syntax_arguments s (print_lident info)) [pv_name rho]
-                (print_expr info 15) e
+                (syntax_arguments s (print_expr info 14)) [e1]
+                (print_expr info 15) e2
             | None ->
                fprintf fmt "@[<hv 2>%a.%a <-@ %a@]"
-                 (print_lident info) (pv_name rho) (print_lident info) rs.rs_name
-                 (print_expr info 15) e in
+                 (print_expr info 14) e1
+                 (print_lident info) rs.rs_name
+                 (print_expr info 15) e2 in
         begin match al with
         | [] -> assert false | [a] -> assign fmt a
         | al -> fprintf fmt "@[begin %a end@]" (print_list semi assign) al end
@@ -628,7 +623,7 @@ module Print = struct
           (print_expr ~opr:false ~be:true info 15) e2
           (print_expr ~be:true info 15) e3
     | Eblock [] ->
-        fprintf fmt "()"
+        pp_print_string fmt "()"
     | Eblock [e] ->
         print_expr ~be info prec fmt e
     | Eblock el ->
@@ -649,7 +644,7 @@ module Print = struct
           (print_expr info 18) e1 (print_expr ~opr:false info 18) e2
     | Eraise (xs, e_opt) ->
         print_raise ~paren:(prec < 4) info xs fmt e_opt
-    | Efor (pv1, pv2, dir, pv3, e) ->
+    | Efor (pv1, _, pv2, dir, pv3, e) ->
         if is_mapped_to_int info pv1.pv_ity then begin
           fprintf fmt "@[<hv 2>for %a = %a %a %a do@ @[%a@]@ done@]"
             (print_lident info) (pv_name pv1) (print_lident info) (pv_name pv2)
@@ -720,7 +715,7 @@ module Print = struct
 
   and print_xbranch info case fmt (xs, pvl, e) =
     let print_exn fmt () =
-      if case then fprintf fmt "exception " else fprintf fmt "" in
+      if case then pp_print_string fmt "exception " in
     let print_var fmt pv = print_lident info fmt (pv_name pv) in
     match query_syntax info.info_syn xs.xs_name, pvl with
     | Some s, _ when complex_syntax s || pvl = [] ->
@@ -744,7 +739,7 @@ module Print = struct
       match cs_args with
       | [] -> fprintf fmt "@[<hov 4>| %a@]" (print_uident info) id
       | l -> fprintf fmt "@[<hov 4>| %a of %a@]" (print_uident info) id
-               (print_list star (print_ty ~use_quote:true ~paren:false info)) l in
+               (print_list star (print_ty ~use_quote:true ~paren:true info)) l in
     let print_field fmt (is_mutable, id, ty) =
       fprintf fmt "%s%a: @[%a@];" (if is_mutable then "mutable " else "")
         (print_lident info) id (print_ty ~use_quote:true ~paren:false info) ty in
@@ -822,7 +817,7 @@ module Print = struct
     let print_pair fmt (s, dl) =
       let info = { info with info_current_ph = s :: info.info_current_ph } in
       fprintf fmt "(%s:@ %a)" s (print_sig info) dl in
-    fprintf fmt "%a" (print_list space print_pair) args
+    print_list space print_pair fmt args
 
   let print_decl info fmt decl =
     (* avoids printing the same decl for mutually recursive decls *)
@@ -846,9 +841,8 @@ let print_decl =
       info_literal      = pargs.Pdriver.literal;
       info_current_th   = th;
       info_current_mo   = Some m;
-      info_th_known_map = th.th_known;
       info_mo_known_map = m.mod_known;
-      info_fname        = Opt.map Compile.clean_name fname;
+      info_fname        = Opt.map Ml_printer.clean_name fname;
       info_flat         = flat;
       info_prec         = pargs.Pdriver.prec;
       info_current_ph   = [];
@@ -859,14 +853,21 @@ let print_decl =
 let ng suffix ?fname m =
   let mod_name = m.mod_theory.th_name.id_string in
   let path     = m.mod_theory.th_path in
-  (module_name ?fname path mod_name) ^ suffix
+  (Ml_printer.module_name ?fname path mod_name) ^ suffix
 
 let ocaml_printer = Pdriver.{
     desc = "printer for Ocaml code";
     implem_printer = {
         filename_generator = ng ".ml";
-        decl_printer = print_decl;
-        prelude_printer = dummy_prelude_printer;
+        decl_printer = print_decl ~flat:false;
+        prelude_printer = default_prelude_printer;
+        header_printer = dummy_border_printer;
+        footer_printer = dummy_border_printer;
+      };
+    flat_printer = {
+        filename_generator = ng ".ml";
+        decl_printer = print_decl ~flat:true;
+        prelude_printer = default_prelude_printer;
         header_printer = dummy_border_printer;
         footer_printer = dummy_border_printer;
       };
