@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -40,7 +40,7 @@ let run_on_task fmt prover prover_driver t =
          ~limit
          prover_driver t)
   in
-  Format.fprintf fmt "%a" Call_provers.print_prover_answer result.Call_provers.pr_answer;
+  Call_provers.print_prover_answer fmt result.Call_provers.pr_answer;
   match result.Call_provers.pr_answer with
   | Call_provers.Failure _ | Call_provers.HighFailure ->
     Format.fprintf fmt "@\n=======@\n%s@\n======@\n" result.Call_provers.pr_output
@@ -59,7 +59,7 @@ let get_prover config env acc (short, name) =
   (* loading the driver *)
   let driver : Why3.Driver.driver =
     try
-      Why3.Whyconf.load_driver (Why3.Whyconf.get_main config) env prover.Whyconf.driver []
+      Why3.Whyconf.load_driver (Why3.Whyconf.get_main config) env prover
     with e ->
       ACSLtoWhy3.Self.fatal "Failed to load driver for %s: %a@." name
         Exn_printer.exn_printer e
