@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -18,4 +18,6 @@ val ty_constructors : known_map -> ty -> Decl.constructor list
 val cs_fields : known_map -> lsymbol -> lsymbol option list
 val select_field : lsymbol -> lsymbol option list -> 'a list -> 'a
 
-val eval_match : known_map -> term -> term
+val eval_match : keep_trace:bool -> known_map -> term -> term
+(** when [keep_trace] is set, the symbols introduced for traceability
+    of counterexamples are always kept *)
