@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,6 +9,7 @@
 (*                                                                  *)
 (********************************************************************)
 
+open Mysexplib.Std [@@warning "-33"]
 open Wstdlib
 open Ident
 open Ty
@@ -578,10 +579,10 @@ let mk_flg ~frz ~exp ~lbl ~fxd ~vis = {
   its_frozen = frz; its_exposed = exp; its_liable = lbl;
   its_fixed  = fxd; its_visible = vis }
 
-let its_of_ts ts priv =
+let its_of_ts ts priv ofld =
   assert (ts.ts_def = NoDef);
   let flg = mk_flg ~frz:priv ~exp:true ~lbl:priv ~fxd:true ~vis:true in
-  mk_its ~ts ~nfr:priv ~priv ~mut:false ~frg:false ~mfld:[] ~ofld:[]
+  mk_its ~ts ~nfr:priv ~priv ~mut:false ~frg:false ~mfld:[] ~ofld
     ~regs:[] ~aflg:(List.map (fun _ -> flg) ts.ts_args) ~rflg:[] ~def:NoDef
 
 let create_rec_itysymbol id args =
@@ -765,11 +766,19 @@ let t_freepvs pvs t = pvs_of_vss pvs (t_vars t)
 
 (** builtin symbols *)
 
-let its_int  = its_of_ts ts_int  true
-let its_real = its_of_ts ts_real true
-let its_bool = its_of_ts ts_bool true
-let its_str  = its_of_ts ts_str  true
-let its_func = its_of_ts ts_func true
+let its_of_ts ts =
+  (* the only non-private types are bool and n-tuples *)
+  let priv = not (ts_equal ts ts_bool || is_ts_tuple ts) in
+  (* the only built-in types with fields are n-tuples *)
+  let mk_fld v = create_pvsymbol (id_fresh "a") (ity_var v) in
+  let ofld = if priv then [] else List.map mk_fld ts.ts_args in
+  its_of_ts ts priv ofld
+
+let its_int  = its_of_ts ts_int
+let its_real = its_of_ts ts_real
+let its_bool = its_of_ts ts_bool
+let its_str  = its_of_ts ts_str
+let its_func = its_of_ts ts_func
 
 let ity_int  = ity_app its_int  [] []
 let ity_real = ity_app its_real [] []
@@ -779,7 +788,7 @@ let ity_str  = ity_app its_str  [] []
 let ity_func a b = ity_app its_func [a;b] []
 let ity_pred a   = ity_app its_func [a;ity_bool] []
 
-let its_tuple = Hint.memo 17 (fun n -> its_of_ts (ts_tuple n) false)
+let its_tuple = Hint.memo 17 (fun n -> its_of_ts (ts_tuple n))
 
 let ity_tuple tl = ity_app (its_tuple (List.length tl)) tl []
 
@@ -795,6 +804,7 @@ type mask =
   | MaskVisible
   | MaskTuple of mask list
   | MaskGhost
+[@@deriving sexp_of]
 
 let mask_of_pv v = if v.pv_ghost then MaskGhost else MaskVisible
 
@@ -1162,6 +1172,11 @@ let eff_union e1 e2 =
             (Mreg.set_diff e.eff_writes e.eff_covers));
   e
 
+let eff_fusion e1 e2 =
+  let e = eff_union e1 e2 in
+  {e with eff_writes = Mreg.set_inter e.eff_writes e.eff_covers;
+          eff_taints = Mreg.set_inter e.eff_taints e.eff_covers}
+
 let eff_contaminate e1 e2 =
   if not e1.eff_ghost then e2 else
   if Sxs.is_empty e1.eff_raises then e2 else
@@ -1260,10 +1275,19 @@ let open_post_with t q = match q.t_node with
   | Teps bf -> t_open_bound_with t bf
   | _ -> invalid_arg "Ity.open_post_with"
 
-let clone_post_result q = match q.t_node with
-  | Teps bf -> t_clone_bound_id bf
+let clone_post_result ?loc ?attrs q =
+  match q.t_node with
+  | Teps bf -> t_clone_bound_id ?loc ?attrs bf
   | _ -> invalid_arg "Ity.clone_post_result"
 
+let default_result_name = "result"
+
+let result_id ?loc ?attrs ?(ql=[]) () : preid =
+  match ql with
+  | q :: _ -> clone_post_result ?loc ?attrs q
+  | _ -> id_fresh ?loc ?attrs default_result_name
+
+
 let annot_attr = Ident.create_attribute "vc:annotation"
 let break_attr = Ident.create_attribute "vc:break_me"
 
@@ -1467,7 +1491,7 @@ let cty_apply c vl args res =
 
 let cty_tuple args =
   let ty = ty_tuple (List.map (fun v -> v.pv_vs.vs_ty) args) in
-  let vs = create_vsymbol (id_fresh "result") ty in
+  let vs = create_vsymbol (result_id ()) ty in
   let tl = List.map (fun v -> t_var v.pv_vs) args in
   let post = create_post vs (t_equ (t_var vs) (t_tuple tl)) in
   let mask = mask_reduce (MaskTuple (List.map mask_of_pv args)) in
@@ -1481,7 +1505,7 @@ let cty_exec_post_raw c =
   let ity = List.fold_right (fun a ity ->
     ity_func a.pv_ity ity) c.cty_args c.cty_result in
   let al = List.map (fun a -> a.pv_vs) c.cty_args in
-  let res = create_vsymbol (id_fresh "result") (ty_of_ity ity) in
+  let res = create_vsymbol (result_id ()) (ty_of_ity ity) in
   let res_al = t_func_app_l (t_var res) (List.map t_var al) in
   let oldies = Mpv.fold (fun {pv_vs = o} {pv_vs = v} s ->
     Mvs.add o (t_var v) s) c.cty_oldies Mvs.empty in
@@ -1585,7 +1609,8 @@ let xprinter = create_ident_printer []
 let forget_reg r = forget_id rprinter r.reg_name
 
 let print_reg_name fmt r =
-  fprintf fmt "@@%s" (id_unique rprinter r.reg_name)
+  (* `%@` prints a single `@` *)
+  fprintf fmt "%@%s" (id_unique rprinter r.reg_name)
 
 let print_args pr fmt tl = if tl <> [] then
   fprintf fmt "@ %a" (Pp.print_list Pp.space pr) tl
@@ -1705,7 +1730,7 @@ let print_spec args pre post xpost oldies eff fmt ity =
   let print_post fmt q =
     let v, q = open_post q in
     let n = asprintf "%a" print_vs v in
-    if n = "result" || t_v_occurs v q = 0 then
+    if n = default_result_name || t_v_occurs v q = 0 then
       fprintf fmt "@\nensures  { @[%a@] }" print_term q else
       fprintf fmt "@\nreturns  { %s ->@ @[%a@] }" n print_term q;
     forget_var v in
