@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -299,10 +299,17 @@ val pvs_of_vss : Spv.t -> 'a Mvs.t -> Spv.t
 
 (** {2 Exception symbols} *)
 
+(** A mask is a generalized ghost information allowing to handle
+   tuples where some components can be ghost and others are not.
+
+   They are used for expressions, including results of programs, and
+   for exceptions *)
+
 type mask =
-  | MaskVisible
-  | MaskTuple of mask list
-  | MaskGhost
+  | MaskVisible            (** fully non-ghost *)
+  | MaskTuple of mask list (** decomposed ghst status for tuples *)
+  | MaskGhost              (** fully ghost *)
+[@@deriving sexp_of]
 
 val mask_ghost : mask -> bool
 
@@ -401,6 +408,7 @@ val eff_ghostify_weak : bool -> effect -> effect     (* only if has no effect *)
 
 val eff_union_seq : effect -> effect -> effect  (* checks for stale variables *)
 val eff_union_par : effect -> effect -> effect  (* no stale-variable check *)
+val eff_fusion    : effect -> effect -> effect  (* drop invalidated writes *)
 
 val mask_adjust : effect -> ity -> mask -> mask
 
@@ -425,7 +433,14 @@ type post = term  (** postcondition: eps result . post_fmla *)
 
 val open_post : post -> vsymbol * term
 val open_post_with : term -> post -> term
-val clone_post_result : post -> preid
+val clone_post_result :
+  ?loc:Loc.position -> ?attrs:Sattr.t -> post -> preid
+val result_id :
+  ?loc:Loc.position -> ?attrs:Sattr.t -> ?ql:post list -> unit -> preid
+(** [result_id ?loc ?ql ()] returns a fresh pre-identifier whose name
+   is appropriately chosen with respect to the optionally given list
+   of post-conditions (["result"] by default, as hard-coded in this
+   module implementation). *)
 
 val create_post : vsymbol -> term -> post
 
@@ -529,6 +544,7 @@ val forget_cty : cty -> unit      (* forget arguments and oldies *)
 
 val print_its : Format.formatter -> itysymbol -> unit (* type symbol *)
 val print_reg : Format.formatter -> region -> unit    (* region *)
+val print_reg_name : Format.formatter -> region -> unit    (* region name *)
 val print_ity : Format.formatter -> ity -> unit       (* individual type *)
 val print_ity_full : Format.formatter -> ity -> unit  (* type with regions *)
 
