@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -23,7 +23,7 @@ type its_defn = {
   itd_fields       : rsymbol list;
   itd_constructors : rsymbol list;
   itd_invariant    : term list;
-  itd_witness      : expr list;
+  itd_witness      : expr option;
 }
 
 let mk_itd s f c i w = {
@@ -35,13 +35,13 @@ let mk_itd s f c i w = {
 }
 
 let create_alias_decl id args ity =
-  mk_itd (create_alias_itysymbol id args ity) [] [] [] []
+  mk_itd (create_alias_itysymbol id args ity) [] [] [] None
 
 let create_range_decl id ir =
-  mk_itd (create_range_itysymbol id ir) [] [] [] []
+  mk_itd (create_range_itysymbol id ir) [] [] [] None
 
 let create_float_decl id fp =
-  mk_itd (create_float_itysymbol id fp) [] [] [] []
+  mk_itd (create_float_itysymbol id fp) [] [] [] None
 
 let check_field stv f =
   let loc = f.pv_vs.vs_name.id_loc in
@@ -86,16 +86,17 @@ let create_plain_record_decl ~priv ~mut id args fdl invl witn =
   let csl = if priv then [] else if invl <> [] then
     [create_semi_constructor cid s fdl pjl invl] else
     [create_constructor ~constr:1 cid s fdl] in
-  if witn <> [] then begin
-    List.iter2 (fun fd ({e_loc = loc} as e) ->
+  if witn <> None then begin
+    let ty = Ty.ty_tuple @@ List.map (fun fd -> fd.pv_vs.vs_ty) fdl in
+    Opt.iter (fun ({e_loc = loc} as e) ->
       if diverges e.e_effect.eff_oneway then Loc.errorm ?loc
         "This expression may not terminate, it cannot be a witness";
       if partial e.e_effect.eff_oneway then Loc.errorm ?loc
         "This expression may fail, it cannot be a witness";
-      if not (eff_pure e.e_effect) then Loc.errorm ?loc
-        "This expression has side effects, it cannot be a witness";
-      let ety = ty_of_ity e.e_ity and fty = fd.pv_vs.vs_ty in
-      Loc.try2 ?loc ty_equal_check ety fty) fdl witn
+      if not (Sxs.is_empty e.e_effect.eff_raises) then Loc.errorm ?loc
+        "This expression may raise exceptions, it cannot be a witness";
+      let ety = ty_of_ity e.e_ity in
+      Loc.try2 ?loc ty_equal_check ety ty) witn
   end;
   mk_itd s pjl csl invl witn
 
@@ -107,7 +108,7 @@ let create_rec_record_decl s fdl =
   List.iter (check_field (Stv.of_list s.its_ts.ts_args)) fdl;
   let cs = create_constructor ~constr:1 cid s fdl in
   let pjl = List.map (create_projection s) fdl in
-  mk_itd s pjl [cs] [] []
+  mk_itd s pjl [cs] [] None
 
 let create_variant_decl exn get_its csl =
   (* named projections are the same in each constructor *)
@@ -127,7 +128,7 @@ let create_variant_decl exn get_its csl =
   (* and now we can create the type symbol and the constructors *)
   let s = get_its (List.map get_fds csl) and constr = List.length csl in
   let mk_cs (id, fdl) = create_constructor ~constr id s (List.map snd fdl) in
-  mk_itd s (List.map (create_projection s) pjl) (List.map mk_cs csl) [] []
+  mk_itd s (List.map (create_projection s) pjl) (List.map mk_cs csl) [] None
 
 let create_plain_variant_decl id args csl =
   let exn = Invalid_argument "Pdecl.create_plain_variant_decl" in
@@ -291,7 +292,7 @@ let get_syms node pure =
         let add_fd syms s = syms_ity syms s.rs_cty.cty_result in
         let add_cs syms s = syms_pvl syms s.rs_cty.cty_args in
         let syms = List.fold_left add_fd syms d.itd_fields in
-        let syms = List.fold_left syms_expr syms d.itd_witness in
+        let syms = Opt.fold syms_expr syms d.itd_witness in
         List.fold_left add_cs syms d.itd_constructors in
       List.fold_left syms_itd syms dl
   | PDlet ld ->
@@ -585,27 +586,27 @@ open Theory
 let pd_int, pd_real, pd_str, pd_equ = match builtin_theory.th_decls with
   | [{td_node = Decl di}; {td_node = Decl dr};
      {td_node = Decl ds}; {td_node = Decl de}] ->
-      mk_decl (PDtype [mk_itd its_int  [] [] [] []]) [di],
-      mk_decl (PDtype [mk_itd its_real [] [] [] []]) [dr],
-      mk_decl (PDtype [mk_itd its_str  [] [] [] []]) [ds],
+      mk_decl (PDtype [mk_itd its_int  [] [] [] None]) [di],
+      mk_decl (PDtype [mk_itd its_real [] [] [] None]) [dr],
+      mk_decl (PDtype [mk_itd its_str  [] [] [] None]) [ds],
       mk_decl PDpure [de]
   | _ -> assert false
 
 let pd_func, pd_func_app = match highord_theory.th_decls with
   | [{td_node = Decl df}; {td_node = Decl da}] ->
-      mk_decl (PDtype [mk_itd its_func [] [] [] []]) [df],
+      mk_decl (PDtype [mk_itd its_func [] [] [] None]) [df],
       mk_decl (PDlet ld_func_app) [da]
   | _ -> assert false
 
 let pd_bool = match bool_theory.th_decls with
   | [{td_node = Decl db}] ->
-      mk_decl (PDtype [mk_itd its_bool [] [rs_true; rs_false] [] []]) [db]
+      mk_decl (PDtype [mk_itd its_bool [] [rs_true; rs_false] [] None]) [db]
   | _ -> assert false
 
 let pd_tuple = Wstdlib.Hint.memo 17 (fun n ->
   match (tuple_theory n).th_decls with
   | [{td_node = Decl dt}] ->
-      mk_decl (PDtype [mk_itd (its_tuple n) [] [rs_tuple n] [] []]) [dt]
+      mk_decl (PDtype [mk_itd (its_tuple n) [] [rs_tuple n] [] None]) [dt]
   | _ -> assert false)
 
 (** {2 Known identifiers} *)
@@ -693,7 +694,7 @@ let check_match kn d =
   | PDtype dl ->
       List.iter (fun d ->
         List.iter check_term d.itd_invariant;
-        List.iter check_expr d.itd_witness) dl
+        Opt.iter check_expr d.itd_witness) dl
   | PDlet ld -> check_let_defn ld
   | PDexn _ | PDpure -> ()
 
