@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -50,9 +50,35 @@ let pv_is_unit v = ity_equal v.pv_ity ity_unit
 
 let pv_of_ity s ity = create_pvsymbol (id_fresh s) ity
 
-let res_of_post ity ql = create_pvsymbol (result_id ~ql ()) ity
 
-let res_of_cty cty = res_of_post cty.cty_result cty.cty_post
+(**)
+let _print_pv_attr fmt v =
+  Format.fprintf fmt "@[%a@ %a@]" Ity.print_pv v
+    Pretty.print_id_attrs v.pv_vs.vs_name
+(**)
+
+let model_trace_result_attribute = create_model_trace_attr "result"
+
+let explicit_result loc attrs ce ity =
+  let name = match ce.c_node with
+    | Capp (rs, _) ->
+       Format.asprintf "%a'result" Ident.print_decoded rs.rs_name.id_string
+    | Cpur (ls, _) ->
+       Format.asprintf "%a'result" Ident.print_decoded ls.ls_name.id_string
+    | Cfun _ -> "anonymous'result"
+    | Cany -> "any'result"
+  in
+  let attrs = Sattr.filter (fun a -> Ident.get_call_id_value a <> None) attrs in
+  let attrs = Sattr.add model_trace_result_attribute attrs in
+  let attrs = match loc with
+    | Some l -> Sattr.add (create_call_result_attr l) attrs
+    | None -> attrs in
+  create_pvsymbol (id_fresh ?loc ~attrs name) ity
+
+
+let res_of_post loc ity ql = create_pvsymbol (result_id ?loc ~ql ()) ity
+
+let res_of_cty loc cty = res_of_post loc cty.cty_result cty.cty_post
 
 let proxy_of_expr =
   let attrs = Sattr.singleton proxy_attr in fun e ->
@@ -65,6 +91,10 @@ let wb_attr = Ident.create_attribute "vc:white_box"
 let kp_attr = Ident.create_attribute "vc:keep_precondition"
 let nt_attr = Ident.create_attribute "vc:divergent"
 
+let do_not_keep_trace_attr = Ident.create_attribute "vc:do_not_keep_trace"
+let do_not_keep_trace_flag = Debug.register_flag "vc:do_not_keep_trace"
+    ~desc:"Do@ not@ keep@ trace@ variables@ in@ model"
+
 let vc_attrs =
   Sattr.add nt_attr (Sattr.add kp_attr (Sattr.add wb_attr
   (Sattr.add sp_attr (Sattr.add wp_attr Sattr.empty))))
@@ -84,9 +114,22 @@ type vc_env = {
   exn_count : int ref;
   divergent : bool;
   inferinvs : (expr * term) list;   (* inferred invariants *)
+  keep_trace : bool;
 }
 
-let mk_env {Theory.th_export = ns_int} {Theory.th_export = ns_acc} kn tuc invs = {
+let mk_env ?(attrs=Sattr.empty)
+      {Theory.th_export = ns_int} {Theory.th_export = ns_acc} kn tuc invs =
+  (* generate traceability info for counterexamples if asked *)
+  let keep_trace =
+    if Debug.test_flag do_not_keep_trace_flag ||
+       Sattr.mem do_not_keep_trace_attr attrs then
+      begin
+        Debug.dprintf debug_vc "keep_trace is unset@.";
+        false
+      end
+    else true
+  in
+  {
   known_map = kn;
   ts_ranges = tuc.Theory.uc_ranges;
   ps_int_le = Theory.ns_find_ls ns_int [Ident.op_infix "<="];
@@ -99,6 +142,7 @@ let mk_env {Theory.th_export = ns_int} {Theory.th_export = ns_acc} kn tuc invs =
   exn_count = ref 0;
   divergent = false;
   inferinvs = invs;
+  keep_trace;
 }
 
 let acc env r t =
@@ -117,10 +161,10 @@ let new_exn env = incr env.exn_count; !(env.exn_count)
 (* FIXME: cannot verify int.why because of a cyclic dependency.
    int.Int is used for the "for" loops and for integer variants.
    We should be able to extract the necessary lsymbols from kn. *)
-let mk_env env kn tuc invs =
+let mk_env ?attrs env kn tuc invs =
   let th_int = Env.read_theory env ["int"] "Int" in
   let th_wf  = Env.read_theory env ["relations"] "WellFounded" in
-  mk_env th_int th_wf kn tuc invs
+  mk_env ?attrs th_int th_wf kn tuc invs
 
 let int_of_range env ty =
   let td = Mts.find ty env.ts_ranges in
@@ -147,6 +191,9 @@ let expl_variant   = Ident.create_attribute "expl:variant decrease"
 let expl_type_inv  = Ident.create_attribute "expl:type invariant"
 let expl_divergent = Ident.create_attribute "expl:termination"
 
+let print_expl fmt expl =
+  Format.pp_print_string fmt (Strings.remove_prefix "expl:" expl.attr_string)
+
 let attrs_has_expl attrs =
   Sattr.exists (fun a -> Strings.has_prefix "expl:" a.attr_string) attrs
 
@@ -219,14 +266,18 @@ let wp_case t bl =
 let wp_forall vl wp = t_forall_close_simp vl [] wp
 let sp_exists vl sp = t_exists_close_simp vl [] sp
 
-let wp_let v t wp =
-  if pv_is_unit v then t_subst_single v.pv_vs t_void wp
-                  else t_let_close_simp v.pv_vs t wp
+let t_let_close_simp env v t f =
+  let keep = env.keep_trace && relevant_for_counterexample v.vs_name in
+  t_let_close_simp_keep_var ~keep v t f
 
-let sp_let v t sp rd =
+let wp_let env v t wp =
+  if pv_is_unit v then t_subst_single v.pv_vs t_void wp
+                  else t_let_close_simp env v.pv_vs t wp
+
+let sp_let env v t sp rd =
   if pv_is_unit v then t_subst_single v.pv_vs t_void sp else
   if Spv.mem v rd then sp_and (t_equ (t_var v.pv_vs) t) sp else
-  t_let_close_simp v.pv_vs t sp
+  t_let_close_simp env v.pv_vs t sp
 
 (* variant decrease preconditions *)
 
@@ -307,8 +358,8 @@ let wp_of_inv loc attrs expl pl =
 
 let wp_of_pre loc attrs pl = wp_of_inv loc attrs expl_pre pl
 
-let wp_of_post expl ity ql =
-  let v = res_of_post ity ql in let t = t_var v.pv_vs in
+let wp_of_post expl loc ity ql =
+  let v = res_of_post loc ity ql in let t = t_var v.pv_vs in
   let make q = vc_expl None Sattr.empty expl (open_post_with t q) in
   v, t_and_asym_l (List.map make ql)
 
@@ -335,7 +386,7 @@ let sp_of_post loc attrs expl v ql = let t = t_var v.pv_vs in
    logical definitions in Pdecl.create_let_decl) *)
 let cty_enrich_post c = match c with
   | {c_node = Cfun e; c_cty = cty} ->
-      let {pv_vs = u} = res_of_cty cty in
+      let {pv_vs = u} = res_of_cty e.e_loc cty in
       let prop = ty_equal u.vs_ty ty_bool in
       begin match term_of_expr ~prop e with
       | Some f ->
@@ -397,7 +448,16 @@ let rec k_print fmt k = match k with
         "@[<hov 4>| %a ->@ %a@]" Pretty.print_pat p k_print k in
       Format.fprintf fmt "@[CASE %a\n@[%a@]@]"
         Ity.print_pv v (Pp.print_list Pp.newline branch) bl
-  | Khavoc _ -> Format.fprintf fmt "HAVOC" (* TODO *)
+  | Khavoc (m, _loc, _sattr) ->
+     Format.fprintf fmt "@[HAVOC @[<hov 2>{ ";
+     Mreg.iter (fun reg pvs ->
+         Format.fprintf fmt "%a -> @[<hov 2>{ " print_reg reg;
+         Mpv.iter (fun pv1 pv2 ->
+             Format.fprintf fmt "%a -> [%a];@ "print_pv pv1 (Pp.print_option print_pv) pv2)
+           pvs;
+         Format.fprintf fmt "}@]")
+       m;
+     Format.fprintf fmt "}@]@]"
   | Klet (v, t, {t_node = Ttrue}) -> Format.fprintf fmt
       "@[<hov 4>LET %a = %a@]" Ity.print_pv v Pretty.print_term t
   | Klet (v,t,f) -> Format.fprintf fmt
@@ -412,7 +472,7 @@ let rec k_print fmt k = match k with
       "@[<hov 4>ASSERT %a@]" Pretty.print_term f
   | Kstop f -> Format.fprintf fmt
       "@[<hov 4>STOP %a@]" Pretty.print_term f
-  | Kcont 0 -> Format.fprintf fmt "SKIP"
+  | Kcont 0 -> Format.pp_print_string fmt "SKIP"
   | Kcont i -> Format.fprintf fmt "RAISE %d" i
   | Kaxiom k -> Format.fprintf fmt "@[<hov 4>AXIOM %a@]" k_print k
   | Ktag (WP, k) -> Format.fprintf fmt "@[<hov 4>WP %a@]" k_print k
@@ -428,7 +488,7 @@ let rec k_print fmt k = match k with
    We need this for simple conjunctions, disjunctions, and
    pattern-matching exprs, to avoid considering each branch
    separately; also to have a single substitutable term. *)
-let term_of_kode res k =
+let term_of_kode env res k =
   let rec get_stack st k = match k with
     | Klet (v, t, f) when pv_equal v res -> st, Some (t, f), 0, Kcont 0
     | Klet (v, t, _) -> (v, Some t) :: st, None, 0, k
@@ -453,7 +513,7 @@ let term_of_kode res k =
   match d with
   | Some (t, f) ->
       let unwind t ({pv_vs = v}, d) = match d with
-        | Some d -> t_let_close_simp v d t
+        | Some d -> t_let_close_simp env v d t
         | None when t_v_occurs v t > 0 -> raise Exit
         | None -> t in
       let t = List.fold_left unwind t st in
@@ -525,6 +585,7 @@ let add_loc_attr label loc attrs =
    - [i] is a positive int assigned at the catching site
    - [xres] names the value carried by the exception *)
 let rec k_expr env lps e res xmap =
+  (* Format.eprintf "[Vc.k_expr] res = %a@." print_pv_attr res; *)
   let loc = e.e_loc and eff = e.e_effect in
   let attrs = Sattr.diff e.e_attrs vc_attrs in
   let t_tag t = t_attr_set ?loc attrs t in
@@ -577,7 +638,7 @@ let rec k_expr env lps e res xmap =
         let xs_lost = Sxs.diff e1.e_effect.eff_raises xs_pass in
         let xq_lost = Mxs.set_inter cty.cty_xpost xs_lost in
         let xq_lost = Mxs.mapi (fun xs ql ->
-          let v = res_of_post xs.xs_ity ql in
+          let v = res_of_post loc xs.xs_ity ql in
           let xq = k_of_post expl_xpost v ql in
           (new_exn env, v), Kseq (xq, 0, bot)) xq_lost in
         (* complete xmap with new indices, then handle e1 *)
@@ -595,6 +656,7 @@ let rec k_expr env lps e res xmap =
         (* [ VC(ce) (if ce is a lambda executed in-place)
            | STOP pre
            | HAVOC ; [ ASSUME post | ASSUME xpost ; RAISE ] ] *)
+       (* Format.eprintf "[Vc.term_of_post/Eexec] res = %a@." print_pv_attr res; *)
         let p, (oldies, sbs) = match pre with
           (* for recursive calls, compute the 'variant decrease'
              precondition and rename the oldies to avoid clash *)
@@ -606,7 +668,7 @@ let rec k_expr env lps e res xmap =
           | pl -> wp_of_pre loc attrs pl, (oldies, Mvs.empty) in
         let trusted = match ce.c_node with
           | (Capp ({rs_logic = RLls ls}, _) | Cpur (ls, _))
-            when ce.c_cty.cty_args = [] (* fully applied *) ->
+               when ce.c_cty.cty_args = [] (* fully applied *) ->
               Typeinv.is_trusted_constructor env.known_map ls ||
               Typeinv.is_trusted_projection env.known_map ls e.e_ity
           | _ -> false in
@@ -615,14 +677,28 @@ let rec k_expr env lps e res xmap =
         let pinv = if trusted then [] else inv_of_pvs env e.e_loc rds in
         let qinv = if trusted then [] else inv_of_pvs env e.e_loc aff in
         let k_of_post expl v ql =
-          let sp = sp_of_post loc attrs expl v ql in
-          let sp = t_subst sbs sp (* rename oldies *) in
-          let rinv = if trusted then [] else
-            inv_of_pvs env e.e_loc (Spv.singleton v) in
-          match term_of_post ~prop:false v.pv_vs sp with
-          | Some (t, sp) ->
-              Klet (v, t_tag t, List.fold_right sp_and rinv sp)
-          | None ->  Kval ([v], List.fold_right sp_and rinv sp) in
+          let k v =
+            let sp = sp_of_post loc attrs expl v ql in
+            let sp = t_subst sbs sp (* rename oldies *) in
+            let rinv = if trusted then [] else
+                         inv_of_pvs env e.e_loc (Spv.singleton v) in
+            match term_of_post ~prop:false v.pv_vs sp with
+            | Some (t, sp) ->
+               Klet (v, t_tag t, List.fold_right sp_and rinv sp)
+            | None ->  Kval ([v], List.fold_right sp_and rinv sp) in
+          let need_trace = match ce.c_node with
+            | (Capp ({rs_logic = RLls _ls}, _) (* `let function` or `val function` *)
+               | Cpur (_ls, _)) (* direct application of a logic symbol *)
+              ->
+               ce.c_cty.cty_args <> [] (* unless not fully applied *)
+          | _ -> true
+            in
+          if env.keep_trace && need_trace then
+            let vv = explicit_result loc e.e_attrs ce v.pv_ity in
+            Kseq(k v,0,Klet(vv, t_var v.pv_vs, t_true))
+          else
+            k v
+          in
         let k = k_of_post expl_post res cty.cty_post in
         (* in abstract blocks, exceptions without postconditions
            escape from the block into the outer code. Otherwise,
@@ -684,8 +760,9 @@ let rec k_expr env lps e res xmap =
           with Not_found -> acc in
         Kseq (Khavoc (Mreg.fold add_write wr Mreg.empty, loc, attrs), 0, k_unit res)
     | Elet (LDvar (v, e0), e1) ->
-        let k = k_expr env lps e1 res xmap in
-        Kseq (k_expr env lps e0 v xmap, 0, k)
+       let k = k_expr env lps e1 res xmap in
+       (* Format.eprintf "[Vc.k_expr/Elet] v = %a@." print_pv_attr v; *)
+       Kseq (k_expr env lps e0 v xmap, 0, k)
     | Elet ((LDsym _| LDrec _) as ld, e1) ->
         let k = k_expr env lps e1 res xmap in
         (* when we havoc the VC of a locally defined function,
@@ -706,8 +783,8 @@ let rec k_expr env lps e res xmap =
           | RLls _ -> assert false (* not applicable *)
           | RLnone -> vl, k
           | RLlemma ->
-              let v = res_of_cty c.c_cty and q = c.c_cty.cty_post in
-              let q = sp_of_post None Sattr.empty expl_post v q in
+             let v = res_of_cty loc c.c_cty and q = c.c_cty.cty_post in
+             let q = sp_of_post None Sattr.empty expl_post v q in
               let q = if pv_is_unit v
                 then t_subst_single v.pv_vs t_void q
                 else t_exists_close_simp [v.pv_vs] [] q in
@@ -745,8 +822,8 @@ let rec k_expr env lps e res xmap =
           if s then try
             if not (ity_equal e.e_ity ity_bool) ||
               ity_fragile e.e_ity then raise Exit;
-            let t1, f1, k1 = term_of_kode res k1 in
-            let t2, f2, k2 = term_of_kode res k2 in
+            let t1, f1, k1 = term_of_kode env res k1 in
+            let t2, f2, k2 = term_of_kode env res k2 in
             let test = t_equ (t_var v.pv_vs) t_bool_true in
             (* with both branches simple, define a resulting term *)
             let t = t_if_simp test t1 t2 and f = sp_if test f1 f2 in
@@ -850,7 +927,8 @@ let rec k_expr env lps e res xmap =
         let j = List.fold_right assert_inv iinv (Kstop init) in
         let k = List.fold_right assert_inv iinv (Kstop keep) in
         let k = Kseq (k_expr env lps e1 res xmap, 0, k) in
-        let k = var_or_proxy e0 (fun v -> Kif (v, k, k_unit res)) in
+        let k = if is_e_true e0 then k
+          else var_or_proxy e0 (fun v -> Kif (v, k, k_unit res)) in
         let k = Kseq (Kval ([], prev), 0, bind_oldies oldies k) in
         let k = List.fold_right assume_inv iinv k in
         let k = check_divergence k in
@@ -908,10 +986,10 @@ let rec k_expr env lps e res xmap =
 
 and k_fun env lps ?(oldies=Mpv.empty) ?(xmap=Mxs.empty) cty e =
   (* ASSUME pre ; LET o = arg ; TRY e ; STOP post WITH STOP xpost *)
-  let res, q = wp_of_post expl_post cty.cty_result cty.cty_post in
+  let res, q = wp_of_post expl_post e.e_loc cty.cty_result cty.cty_post in
   let xq = complete_xpost cty e.e_effect xmap in
   let xq = Mxs.mapi (fun xs ql ->
-    let v, xq = wp_of_post expl_xpost xs.xs_ity ql in
+    let v, xq = wp_of_post expl_xpost e.e_loc xs.xs_ity ql in
     (new_exn env, v), xq) xq in
   let xmap = Mxs.set_union (Mxs.map fst xq) xmap in
   let rds = List.fold_right Spv.add cty.cty_args cty.cty_effect.eff_reads in
@@ -1316,17 +1394,16 @@ let wrt_exists vl f = wrt_rename sp_exists vl f
    the range of [wr_i], or otherwise in [rdm(i)]. Every
    variable in the range of [wr_i] is free in [sp_i]. *)
 
-let rec sp_expr kn k rdm dst = match k with
-  | Kseq (Klet (v, t, f), 0, k2)
-    when not (has_a_model_attr v.pv_vs.vs_name) ->
-      let wp2, sp2, rd2 = sp_expr kn k2 rdm dst in
+let rec sp_expr env k rdm dst = match k with
+  | Kseq (Klet (v, t, f), 0, k2) ->
+      let wp2, sp2, rd2 = sp_expr env k2 rdm dst in
       let rd1 = t_freepvs (t_freepvs rd2 t) f in
-      let wp = wp_let v t (sp_implies f wp2) in
+      let wp = wp_let env v t (sp_implies f wp2) in
       let close _ (sp, wr) rd =
-        Some (sp_let v t (sp_and f sp) rd, wr) in
+        Some (sp_let env v t (sp_and f sp) rd, wr) in
       wp, Mint.inter close sp2 rdm, Spv.remove v rd1
   | Kseq (k1, i, k2) ->
-      let wp2, sp2, rd2 = sp_expr kn k2 rdm dst in
+      let wp2, sp2, rd2 = sp_expr env k2 rdm dst in
       (* log new "written" variables added to dst *)
       let new_written = ref [] in
       let mk_written v =
@@ -1344,7 +1421,7 @@ let rec sp_expr kn k rdm dst = match k with
       let fresh_wr2 v _ = mk_written v in
       let fresh_rd2 v _ = if v.pv_ity.ity_pure then None
                           else Some (mk_written v) in
-      let wp1, sp1, rd1 = sp_expr kn k1 (Mint.add i rd2 rdm)
+      let wp1, sp1, rd1 = sp_expr env k1 (Mint.add i rd2 rdm)
         (Mpv.set_union (Mpv.set_union (Mpv.mapi fresh_wr2 wr2)
         (Mpv.mapi_filter fresh_rd2 (Mpv.set_diff rd2 dst))) dst) in
       (* retrieve the postcondition and the write effect for the
@@ -1390,13 +1467,13 @@ let rec sp_expr kn k rdm dst = match k with
       List.iter (Hvs.remove ht_written) !new_written;
       wp_and wp1 wp2, sp_combine_map sp1 sp2, rd1
   | Kpar (k1, k2) ->
-      let wp1, sp1, rd1 = sp_expr kn k1 rdm dst in
-      let wp2, sp2, rd2 = sp_expr kn k2 rdm dst in
+      let wp1, sp1, rd1 = sp_expr env k1 rdm dst in
+      let wp2, sp2, rd2 = sp_expr env k2 rdm dst in
       wp_and wp1 wp2, sp_combine_map sp1 sp2, Spv.union rd1 rd2
   | Kif (v, k1, k2) ->
       let test = t_equ (t_var v.pv_vs) t_bool_true in
-      let wp1, sp1, rd1 = sp_expr kn k1 rdm dst in
-      let wp2, sp2, rd2 = sp_expr kn k2 rdm dst in
+      let wp1, sp1, rd1 = sp_expr env k1 rdm dst in
+      let wp2, sp2, rd2 = sp_expr env k2 rdm dst in
       let join _ sp1 sp2 = match sp1, sp2 with
         | Some (sp1, wr1), Some (sp2, wr2) ->
             let sp1 = sp_complete sp1 wr1 wr2 in
@@ -1410,7 +1487,7 @@ let rec sp_expr kn k rdm dst = match k with
   | Kcase (v, bl) ->
       let t = t_var v.pv_vs in
       let branch (p, k) (wpl, spl, wrm, rds) =
-        let wp, sp, rd = sp_expr kn k rdm dst in
+        let wp, sp, rd = sp_expr env k rdm dst in
         let wpl = t_close_branch p wp :: wpl in
         let spl = (p, sp) :: spl in
         let join _ wr1 wr2 = Some (Mpv.set_union wr1 wr2) in
@@ -1433,8 +1510,8 @@ let rec sp_expr kn k rdm dst = match k with
   | Khavoc (wr, loc, attrs) ->
       let rd = Mint.find 0 rdm in
       let dst = Mpv.set_inter dst (pvs_affected wr rd) in
-      if Mpv.is_empty dst then sp_expr kn (Kcont 0) rdm dst else
-      let regs = name_regions kn wr dst in
+      if Mpv.is_empty dst then sp_expr env (Kcont 0) rdm dst else
+      let regs = name_regions env.known_map wr dst in
       let () = print_dst dst; print_regs regs in
       let add _ t fvs = t_freevars fvs t in
       let fvs = Mreg.fold add regs Mvs.empty in
@@ -1442,7 +1519,7 @@ let rec sp_expr kn k rdm dst = match k with
       let loc_attr = wrt_mk_loc_attr loc attrs in
       let update {pv_vs = o; pv_ity = ity} n sp =
         wrt_add_loc_attr n loc_attr;
-        let t, fl = havoc kn wr regs (t_var o) ity [] in
+        let t, fl = havoc env.known_map wr regs (t_var o) ity [] in
         sp_and (t_and_l (cons_t_simp (t_var n) t fl)) sp in
       let sp = Mpv.fold update dst t_true in
       let sp = sp_exists (Mvs.keys fvs) sp in
@@ -1454,7 +1531,7 @@ let rec sp_expr kn k rdm dst = match k with
       t_true, Mint.singleton 0 (sp, dst), rd
   | Klet (v, t, f) ->
       let rd = Mint.find 0 rdm in
-      let sp = sp_let v t f rd in
+      let sp = sp_let env v t f rd in
       let rd = Spv.remove v (t_freepvs rd sp) in
       t_true, Mint.singleton 0 (sp, Mpv.empty), rd
   | Kval (vl, f) ->
@@ -1471,50 +1548,50 @@ let rec sp_expr kn k rdm dst = match k with
   | Kcont i ->
       t_true, Mint.singleton i (t_true, Mpv.empty), Mint.find i rdm
   | Kaxiom k ->
-      let f = wp_expr kn k Mint.empty in
+      let f = wp_expr env k Mint.empty in
       let f = vc_expl None Sattr.empty expl_lemma f in
       let rd = t_freepvs (Mint.find 0 rdm) f in
       t_true, Mint.singleton 0 (f, Mpv.empty), rd
   | Ktag (Off expl, k) ->
-      let wp, sp, rd = sp_expr kn k rdm dst in
+      let wp, sp, rd = sp_expr env k rdm dst in
       wp_solder expl wp, sp, rd
   | Ktag (Out out, k) ->
-      sp_expr kn k (Mint.set_inter rdm out) dst
+      sp_expr env k (Mint.set_inter rdm out) dst
   | Ktag (WP, k) ->
       assert (Mint.is_empty rdm);
-      let f = wp_expr kn k Mint.empty in
+      let f = wp_expr env k Mint.empty in
       f, Mint.empty, t_freepvs Spv.empty f
   | Ktag ((Push _|SP), _) -> assert false (* cannot happen *)
 
-and wp_expr kn k q = match k with
+and wp_expr env k q = match k with
   | Kseq (k1, i, k2) ->
-      wp_expr kn k1 (Mint.add i (wp_expr kn k2 q) q)
+      wp_expr env k1 (Mint.add i (wp_expr env k2 q) q)
   | Kpar (k1, k2) ->
-      wp_and (wp_expr kn k1 q) (wp_expr kn k2 q)
+      wp_and (wp_expr env k1 q) (wp_expr env k2 q)
   | Kif ({pv_vs = v}, k1, k2) ->
       let test = t_equ (t_var v) t_bool_true in
-      wp_if test (wp_expr kn k1 q) (wp_expr kn k2 q)
+      wp_if test (wp_expr env k1 q) (wp_expr env k2 q)
   | Kcase ({pv_vs = v}, bl) ->
-      let branch (p,k) = t_close_branch p (wp_expr kn k q) in
+      let branch (p,k) = t_close_branch p (wp_expr env k q) in
       wp_case (t_var v) (List.map branch bl)
   | Khavoc (wr, loc, attrs) ->
       let q = Mint.find 0 q in
       let dst = dst_of_wp loc attrs wr q in
       if Mpv.is_empty dst then q else
-      let regs = name_regions kn wr dst in
+      let regs = name_regions env.known_map wr dst in
       let () = print_dst dst; print_regs regs in
       let add _ t fvs = t_freevars fvs t in
       let fvs = Mreg.fold add regs Mvs.empty in
       let update {pv_vs = o; pv_ity = ity} n q =
-        let t, fl = havoc kn wr regs (t_var o) ity [] in
+        let t, fl = havoc env.known_map wr regs (t_var o) ity [] in
         if Mvs.mem n fvs then
           sp_implies (t_and_l (cons_t_simp (t_var n) t fl)) q
-        else t_let_close_simp n t (sp_implies (t_and_l fl) q) in
+        else t_let_close_simp env n t (sp_implies (t_and_l fl) q) in
       let q = t_subst (adjustment dst) q in
       let q = Mpv.fold update dst q in
       wp_forall (Mvs.keys fvs) q
   | Klet (v, t, f) ->
-      wp_let v t (sp_implies f (Mint.find 0 q))
+      wp_let env v t (sp_implies f (Mint.find 0 q))
   | Kval (vl, f) ->
       let q = sp_implies f (Mint.find 0 q) in
       wp_forall (List.map (fun v -> v.pv_vs) vl) q
@@ -1525,16 +1602,16 @@ and wp_expr kn k q = match k with
   | Kcont i ->
       Mint.find i q
   | Kaxiom k ->
-      let f = wp_expr kn k Mint.empty in
+      let f = wp_expr env k Mint.empty in
       let f = vc_expl None Sattr.empty expl_lemma f in
       sp_implies f (Mint.find 0 q)
   | Ktag (Off expl, k) ->
-      wp_solder expl (wp_expr kn k q)
+      wp_solder expl (wp_expr env k q)
   | Ktag (Out out, k) ->
-      wp_expr kn k (Mint.set_inter q out)
+      wp_expr env k (Mint.set_inter q out)
   | Ktag (SP, k) ->
       let k = Mint.fold (fun i q k -> Kseq (k, i, Kstop q)) q k in
-      let wp, _, _ = sp_expr kn k Mint.empty Mpv.empty in wp
+      let wp, _, _ = sp_expr env k Mint.empty Mpv.empty in wp
   | Ktag ((Push _|WP), _) -> assert false (* cannot happen *)
 
 let rec simp_cast_projections env t = match t.t_node with
@@ -1549,13 +1626,13 @@ let rec simp_cast_projections env t = match t.t_node with
 
 (** VCgen *)
 
-let vc_kode {known_map = kn} vc_wp k =
+let vc_kode env vc_wp k =
   if Debug.test_flag debug_vc then
     Format.eprintf "K @[%a@]@\n" k_print k;
   let k = reflow vc_wp k in
   if Debug.test_flag debug_reflow then
     Format.eprintf "R @[%a@]@\n" k_print k;
-  wp_expr kn k Mint.empty
+  wp_expr env k Mint.empty
 
 let vc_fun env vc_wp cty e =
   vc_kode env vc_wp (k_fun env Mls.empty cty e)
@@ -1563,7 +1640,7 @@ let vc_fun env vc_wp cty e =
 let vc_rec env vc_wp rdl =
   List.map (vc_kode env vc_wp) (k_rec env Mls.empty rdl)
 
-let mk_vc_decl ({known_map = kn} as env) id f =
+let mk_vc_decl ({known_map = kn; keep_trace } as env) id f =
   let {id_string = nm; id_attrs = attrs; id_loc = loc} = id in
   let attrs = if attrs_has_expl attrs then attrs else
     Sattr.add (Ident.create_attribute ("expl:VC for " ^ nm)) attrs in
@@ -1571,12 +1648,32 @@ let mk_vc_decl ({known_map = kn} as env) id f =
   let f = wp_forall (Mvs.keys (t_freevars Mvs.empty f)) f in
   let f = Typeinv.inject kn f in
   let f = if Debug.test_flag debug_no_eval then f else
-          Eval_match.eval_match kn f in
+            begin
+              Debug.dprintf debug_vc "Calling eval_match with keep_trace = %b@." keep_trace;
+              Eval_match.eval_match ~keep_trace kn f
+            end
+  in
   let f = simp_cast_projections env f in
   create_pure_decl (create_prop_decl Pgoal pr f)
 
+let rec is_trivial_vc f =
+  match f.t_node with
+  | Ttrue -> true
+  | Tlet (_,tb) -> let _,t = t_open_bound tb in is_trivial_vc t
+  | Tquant(Tforall,tq) -> let _,_,t = t_open_quant tq in is_trivial_vc t
+  | Tbinop(Timplies,_,t) -> is_trivial_vc t
+  | Tbinop(Tand,t1,t2) -> is_trivial_vc t1 && is_trivial_vc t2
+  | Tcase(_,tbl) ->
+     List.for_all (fun tb -> let _,t = t_open_branch tb in is_trivial_vc t) tbl
+  | Tif(_,t1,t2) -> is_trivial_vc t1 && is_trivial_vc t2
+  | _ -> false
+
+let rec has_attr a f =
+  Sattr.mem annot_attr f.t_attrs || t_any (has_attr a) f
+
 let add_vc_decl kn id f vcl =
-  if can_simp f then vcl else mk_vc_decl kn id f :: vcl
+  if is_trivial_vc f && not (has_attr annot_attr f)
+  then vcl else mk_vc_decl kn id f :: vcl
 
 let infer_invs = ref (fun _ _ _ _ _ _ -> [])
 let set_infer_invs f = infer_invs := f
@@ -1585,40 +1682,51 @@ let vc env kn tuc d = match d.pd_node with
   | PDlet (LDvar (_, {e_node = Eexec ({c_node = Cany},_)})) ->
       []
   | PDlet (LDvar (v, e)) ->
-      let env = mk_env env kn tuc [] in
       let c, e = match e.e_node with
         | Eexec ({c_node = Cfun e} as c, _) -> c, e
         | _ -> c_fun [] [] [] Mxs.empty Mpv.empty e, e in
+      let env = mk_env ~attrs:e.e_attrs env kn tuc [] in
       let f = vc_fun env (Debug.test_noflag debug_sp) c.c_cty e in
       add_vc_decl env v.pv_vs.vs_name f []
   | PDlet (LDsym (s, {c_node = Cfun e; c_cty = cty})) ->
       let open Theory in
       let attrs = s.rs_name.id_attrs in
       let invs = !infer_invs attrs env tuc.uc_known kn e cty in
-      let env = mk_env env kn tuc invs in
+      let env = mk_env ~attrs:e.e_attrs env kn tuc invs in
       let f = vc_fun env (Debug.test_noflag debug_sp) cty e in
       add_vc_decl env s.rs_name f []
   | PDlet (LDrec rdl) ->
+     (** FIXME : pass ~attrs:e.e_attrs *)
       let env = mk_env env kn tuc [] in
       let fl = vc_rec env (Debug.test_noflag debug_sp) rdl in
       let add rd f vcl = add_vc_decl env rd.rec_sym.rs_name f vcl in
       List.fold_right2 add rdl fl []
   | PDtype tdl ->
       let env = lazy (mk_env env kn tuc []) in
-      let add_witness d vcl =
+      let add_witness d wit vcl =
         let env = Lazy.force env in
-        let add_fd (mv,ldl) fd e =
+        let pre_pattern =
+          List.map (fun fd ->
+              let fd = fd_of_rs fd in
+              let id = id_clone fd.pv_vs.vs_name in
+              PPvar(id,fd.pv_ghost))
+            d.itd_fields in
+        let pre_pattern =
+          PPapp(Expr.rs_tuple (List.length d.itd_fields), pre_pattern)
+        in
+        let vars, pattern =
+          Expr.create_prog_pattern pre_pattern wit.e_ity wit.e_mask
+        in
+        let add_fd mv fd =
           let fd = fd_of_rs fd in
-          let id = id_clone fd.pv_vs.vs_name in
-          let ld, v = let_var id ~ghost:fd.pv_ghost e in
-          Mvs.add fd.pv_vs (t_var v.pv_vs) mv, ld::ldl in
-        let mv, ldl = List.fold_left2 add_fd
-          (Mvs.empty, []) d.itd_fields d.itd_witness in
+          let pv = Mstr.find fd.pv_vs.vs_name.id_string vars in
+          Mvs.add fd.pv_vs (t_var pv.pv_vs) mv in
+        let mv = List.fold_left add_fd Mvs.empty d.itd_fields in
         let e = List.fold_right (fun f e ->
           let f = vc_expl None Sattr.empty expl_type_inv (t_subst mv f) in
           let ld, _ = let_var (id_fresh "_") (e_assert Assert f) in
           e_let ld e) d.itd_invariant e_void in
-        let e = List.fold_right e_let ldl e in
+        let e = e_match wit [pattern,e] Mxs.empty in
         let c = c_fun [] [] [] Mxs.empty Mpv.empty e in
         let f = vc_fun env (Debug.test_noflag debug_sp) c.c_cty e in
         add_vc_decl env d.itd_its.its_ts.ts_name f vcl in
@@ -1631,8 +1739,10 @@ let vc env kn tuc d = match d.pd_node with
         let f = t_exists_close_simp vl [] f in
         add_vc_decl env d.itd_its.its_ts.ts_name f vcl in
       let add_itd d vcl =
-        if d.itd_witness <> [] then add_witness d vcl else
-        if d.itd_invariant <> [] then add_invariant d vcl else
-        vcl in
+        match d.itd_witness, d.itd_invariant with
+        | Some wit, _ -> add_witness d wit vcl
+        | None, _::_ -> add_invariant d vcl
+        | None, [] -> vcl
+      in
       List.fold_right add_itd tdl []
   | _ -> []
