@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,19 +9,19 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Mysexplib.Std [@@warning "-33"]
-
-(** Parse trees *)
-
 (** {1 Parse trees} *)
 
+open Mysexplib.Std [@@warning "-33"]
+
 (** {2 Identifiers and attributes} *)
 
+(** attributes, with a specific case for a source location *)
 type attr =
   | ATstr of Ident.attribute
   | ATpos of Loc.position
 [@@deriving sexp_of]
 
+(** identifiers, with attributes and a source location *)
 type ident = {
   id_str : string;
   id_ats : attr list;
@@ -29,6 +29,7 @@ type ident = {
 }
 [@@deriving sexp_of]
 
+(** qualified identifiers *)
 type qualid =
   | Qident of ident
   | Qdot of qualid * ident
@@ -36,22 +37,33 @@ type qualid =
 
 (** {2 Types} *)
 
+(** type expressions *)
 type pty =
   | PTtyvar of ident
+  (** type variable *)
   | PTtyapp of qualid * pty list
+  (** type constructor, possibly with arguments, e.g. [int], [list bool], etc. *)
   | PTtuple of pty list
+  (** tuples, e.g. [(int,bool)] *)
   | PTref   of pty list
+  (** reference type, e.g. [ref (int,bool)] *)
   | PTarrow of pty * pty
+  (** arrow type, e.g. [int -> bool] *)
   | PTscope of qualid * pty
+  (** opening scope locally, e.g. [M.((list t,u))] *)
   | PTparen of pty
+  (** parenthesised type *)
   | PTpure  of pty
+  (** purify a type *)
 [@@deriving sexp_of]
 
+
 (** {2 Patterns} *)
 
 type ghost = bool
 [@@deriving sexp_of]
 
+(** Patterns, equipped with a source location *)
 type pattern = {
   pat_desc : pat_desc;
   pat_loc  : Loc.position;
@@ -59,18 +71,30 @@ type pattern = {
 
 and pat_desc =
   | Pwild
+  (** wildcard, that is [_] *)
   | Pvar of ident
+  (** variable as a pattern *)
   | Papp of qualid * pattern list
+  (** constructor pattern, e.g [Cons(x,y)] *)
   | Prec of (qualid * pattern) list
+  (** record pattern *)
   | Ptuple of pattern list
+  (** tuple pattern *)
   | Pas of pattern * ident * ghost
+  (** as-pattern, e.g. [Cons(x,y) as z] *)
   | Por of pattern * pattern
+  (** or-pattern [p1 | p2] *)
   | Pcast of pattern * pty
+  (** type cast *)
   | Pscope of qualid * pattern
+  (** open scope locally *)
   | Pparen of pattern
+  (** parenthesised pattern *)
   | Pghost of pattern
+  (** explicitly ghost pattern *)
 [@@deriving sexp_of]
 
+
 (** {2 Logical terms and formulas} *)
 
 type binder = Loc.position * ident option * ghost * pty option
@@ -79,6 +103,7 @@ type binder = Loc.position * ident option * ghost * pty option
 type param  = Loc.position * ident option * ghost * pty
 [@@deriving sexp_of]
 
+(** Terms, equipped with a source location *)
 type term = {
   term_desc : term_desc;
   term_loc  : Loc.position;
@@ -86,28 +111,55 @@ type term = {
 
 and term_desc =
   | Ttrue
+  (** the true proposition *)
   | Tfalse
+  (** the false proposition *)
   | Tconst of Constant.constant
+  (** constant literals *)
   | Tident of qualid
+  (** identifiers *)
   | Tasref of qualid
+  (** identifier as reference, e.g. [&x] *)
   | Tidapp of qualid * term list
+  (** (first-order) application of a logic identifier to a list of terms *)
   | Tapply of term * term
+  (** curryied application, of a term to a term *)
   | Tinfix of term * ident * term
+  (** application of a binary operation in an infix fashion, allowing chaining
+      of such. For example, [Tinfix(t1,"<=",Tinfix(t2,"<",t3))] denotes
+      [t1 <= t2 /\ t2 < t3] *)
   | Tinnfix of term * ident * term
+  (** application of a binary operation in an infix style, but without chaining *)
   | Tbinop of term * Dterm.dbinop * term
+  (** application of a binary logic connective, in an infix fashion, allowing
+      chaining. For example, [Tbinop(p1,"<->",Tbinop(p2,"<->",p3))] denotes
+      [(p1 <-> p2) /\ (p2 <-> p3)] *)
   | Tbinnop of term * Dterm.dbinop * term
+  (** application of a binary logic connective, but without chaining *)
   | Tnot of term
+  (** logic negation *)
   | Tif of term * term * term
+  (** if-expression *)
   | Tquant of Dterm.dquant * binder list * term list list * term
+  (** quantified formulas. The third argument is a list of triggers. *)
   | Tattr of attr * term
+  (** term with an attribute *)
   | Tlet of ident * term * term
+  (** let-expression *)
   | Tcase of term * (pattern * term) list
+  (** pattern-matching *)
   | Tcast of term * pty
+  (** type casting *)
   | Ttuple of term list
+  (** tuples *)
   | Trecord of (qualid * term) list
+  (** record expressions *)
   | Tupdate of term * (qualid * term) list
+  (** update-expression *)
   | Tscope of qualid * term
+  (** local scope *)
   | Tat of term * ident
+  (** at modifier *)
 [@@deriving sexp_of]
 
 (** {2 Program expressions} *)
@@ -166,38 +218,41 @@ and expr_desc =
   | Eident of qualid
   (** Variable identifier *)
   | Easref of qualid
-  (** TODO: document *)
+  (** identifier as reference, e.g. [&x] *)
   | Eidapp of qualid * expr list
   (** Uncurried application of a function identifier to a list of arguments *)
   | Eapply of expr * expr
   (** Curried application *)
   | Einfix of expr * ident * expr
-  (** Infix operation *)
+  (** application of a binary function identifier, in an infix fashion, allowing
+     chaining, e.g. [Einfix(e1,"<=",Einfix(e2,"<",e3))] denotes
+     [e1 <= e2 && e2 < e3]  *)
   | Einnfix of expr * ident * expr
-  (** Infix operation (TODO: document the difference with the former) *)
+  (** application of a binary function, but without chaining *)
   | Elet of ident * ghost * Expr.rs_kind * expr * expr
   (** [let ... in ...] expression *)
   | Erec of fundef list * expr
-  (** Local definition of function, possibly mutually recursive *)
+  (** Local definition of function(s), possibly mutually recursive *)
   | Efun of binder list * pty option * pattern * Ity.mask * spec * expr
   (** Anonymous function *)
   | Eany of param list * Expr.rs_kind * pty option * pattern * Ity.mask * spec
   (** [any]: abstract expression with a specification,
-       generating of VC for existence *)
+       generating a VC for existence *)
   | Etuple of expr list
   (** Tuple of expressions *)
   | Erecord of (qualid * expr) list
-  (** Record expressions *)
+  (** Record expressions, e.g. [{f=e1; g=e2; ...}] *)
   | Eupdate of expr * (qualid * expr) list
-  (** Record update [{e with f=e1; ...}] *)
+  (** Record update, e.g. [{e with f=e1; ...}] *)
   | Eassign of (expr * qualid option * expr) list
-  (** Parallel assignment ? TODO: document *)
+  (** assignment, of a mutable variable (no qualid given) or of a record field (qualid
+      given). assignment are possibly in parallel, e.g [x.f, y.g, z <- e1, e2, e3] *)
   | Esequence of expr * expr
   (** Sequence of two expressions *)
   | Eif of expr * expr * expr
   (** [if .. then .. else ..] expression *)
   | Ewhile of expr * invariant * variant * expr
-  (** [while] loop *)
+  (** [while] loop with annotations *)
   | Eand of expr * expr
   (** lazy conjunction *)
   | Eor of expr * expr
@@ -210,27 +265,28 @@ and expr_desc =
   | Eabsurd
   (** [absurd] statement to mark unreachable branches *)
   | Epure of term
-  (** turns a logical term into a pure expression *)
+  (** turns a logical term into a pure expression, e.g [pure { t }] *)
   | Eidpur of qualid
-  (** TODO: document *)
+  (** promotes a logic symbol in programs, e.g [{f}] or [M.{f}] *)
   | Eraise of qualid * expr option
-  (** raise an exception *)
+  (** raise an exception, possibly with an argument *)
   | Eexn of ident * pty * Ity.mask * expr
   (** local declaration of an exception *)
   | Eoptexn of ident * Ity.mask * expr
-  (** TODO: document *)
+  (** local declaration of an exception, implicitly captured. Used by Why3 for handling
+     [return], [break] and [continue] *)
   | Efor of ident * expr * Expr.for_direction * expr * invariant * expr
-  (** [for] loop *)
+  (** [for] loops *)
   | Eassert of Expr.assertion_kind * term
-  (** [assert] expression *)
+  (** [assert], [assume] or [check] expressions *)
   | Escope of qualid * expr
-  (** TODO: document *)
+  (** open scope locally, e.g. [M.(e)] *)
   | Elabel of ident * expr
-  (** introduction of a label *)
+  (** introduction of a label, e.g. [label L in e] *)
   | Ecast of expr * pty
-  (** cast an expression to a given type *)
+  (** cast an expression to a given type, e.g. [(e:ty)] *)
   | Eghost of expr
-  (** forces an expression to be ghost *)
+  (** forces an expression to be ghost, e..g [ghost e] *)
   | Eattr of attr * expr
   (** attach an attribute to an expression *)
 
@@ -283,7 +339,7 @@ type type_decl = {
   td_vis    : visibility; (** visibility, for records only *)
   td_mut    : bool;       (** mutability, for records or abstract types *)
   td_inv    : invariant;  (** invariant, for records only *)
-  td_wit    : (qualid * expr) list;  (** witness for the invariant *)
+  td_wit    : expr option;  (** witness for the invariant *)
   td_def    : type_def;
 }
 [@@deriving sexp_of]
@@ -351,7 +407,7 @@ type decl =
   (** Declaration of global exceptions *)
   | Dmeta of ident * metarg list
   (** Declaration of a [meta] *)
-  | Dcloneexport of qualid * clone_subst list
+  | Dcloneexport of Loc.position * qualid * clone_subst list
   (** [clone export] *)
   | Duseexport of qualid
   (** [use export] *)
