@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -457,16 +457,21 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
         | None -> Loc.error ~loc (RecordFieldMissing pj) in
       let cs, fl = parse_record ~loc ns km get_val fl in
       DTapp (cs, fl)
-  | Ptree.Tupdate (e1, fl) ->
+  | Ptree.Tupdate ({ term_loc = e1_loc } as e1, fl) ->
       let e1 = dterm ns km crcmap gvars at denv e1 in
       let re = is_reusable e1 in
-      let v = if re then e1 else mk_var crcmap "q " e1 in
+      let v = if re then e1 else mk_var crcmap "_q " e1 in
+      let used = ref false in
       let get_val _ pj = function
         | Some e -> dterm ns km crcmap gvars at denv e
-        | None -> Dterm.dterm crcmap ~loc (DTapp (pj,[v])) in
+        | None ->
+            used := true;
+            Dterm.dterm crcmap ~loc (DTapp (pj,[v])) in
       let cs, fl = parse_record ~loc ns km get_val fl in
+      if not !used then
+        Warning.emit ~loc:e1_loc "unused expression (every field is overwritten)";
       let d = DTapp (cs, fl) in
-      if re then d else mk_let crcmap ~loc "q " e1 d
+      if re then d else mk_let crcmap ~loc "_q " e1 d
   | Ptree.Tat (e1, ({id_str = l; id_loc = loc} as id)) ->
       Hstr.add at_uses l false;
       let id = { id with id_str = "" } in
@@ -991,18 +996,22 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
         | Some e -> dexpr muc denv e in
       let cs,fl = parse_record ~loc muc get_val fl in
       expr_app loc (DEsym (RS cs)) fl
-  | Ptree.Eupdate (e1, fl) ->
+  | Ptree.Eupdate ({ expr_loc = e1_loc } as e1, fl) ->
       let e1 = dexpr muc denv e1 in
       let re = is_reusable e1 in
-      let v = if re then e1 else mk_var "q " e1 in
+      let v = if re then e1 else mk_var "_q " e1 in
+      let used = ref false in
       let get_val _ pj = function
         | None ->
+            used := true;
             let pj = Dexpr.dexpr ~loc (DEsym (RS pj)) in
             Dexpr.dexpr ~loc (DEapp (pj, v))
         | Some e -> dexpr muc denv e in
       let cs,fl = parse_record ~loc muc get_val fl in
+      if not !used then
+        Warning.emit ~loc:e1_loc "unused expression (every field is overwritten)";
       let d = expr_app loc (DEsym (RS cs)) fl in
-      if re then d else mk_let ~loc "q " e1 d
+      if re then d else mk_let ~loc "_q " e1 d
   | Ptree.Elet (id, gh, kind, e1, e2) ->
       let e1 = update_any kind e1 in
       let kind = local_kind kind in
@@ -1198,9 +1207,78 @@ let check_public ~loc d name =
     Loc.errorm ~loc "%s types cannot be abstract, private, or mutable" name;
   if d.td_inv <> [] then
     Loc.errorm ~loc "%s types cannot have invariants" name;
-  if d.td_wit <> [] then
+  if d.td_wit <> None then
     Loc.errorm ~loc "%s types cannot have witnesses" name
 
+let type_wit muc nms fl dwit =
+  let convert_record loc tl =
+    let add_w m (q,e) =
+      let v = try match q with
+        | Qident x -> Mstr.find x.id_str nms
+        | Qdot _ -> raise Not_found
+        with Not_found -> Loc.errorm ~loc:(qloc q)
+                            "Unknown field %a" print_qualid q in
+      let e = if v.pv_ghost then
+          { expr_desc = Ptree.Eghost e; expr_loc = e.expr_loc }
+        else e
+      in
+      try Mpv.add_new Exit v e m
+      with Exit ->
+         Loc.errorm ~loc:(qloc q)
+           "Field present twice %a" print_qualid q
+    in
+    let wit = List.fold_left add_w Mpv.empty tl in
+    let wit =
+      List.map (fun (_,v) -> try Mpv.find v wit with
+          | _ -> Loc.errorm ~loc
+                   "Missing field %s" v.pv_vs.vs_name.id_string) fl in
+    Etuple wit
+  in
+  let rec convert e =
+    let add_loc expr_desc = { expr_desc; expr_loc = e.expr_loc } in
+    match e.expr_desc with
+    | Ptree.Eref | Etrue | Efalse | Ptree.Econst _ | Eident _
+    | Easref _
+    | Eidapp (_, _)
+    | Eapply (_, _) | Einfix (_, _, _) | Einnfix (_, _, _)
+    | Efun  (_, _, _, _, _, _) | Eany (_, _, _, _, _, _)
+    | Etuple _
+    | Eupdate (_, _) | Ptree.Eassign _
+    | Ptree.Ewhile (_, _, _, _) | Eand (_, _)
+    | Eor (_, _) | Enot _ | Ptree.Eabsurd | Ptree.Epure _
+    | Eidpur _
+    | Ptree.Eraise (_, _)
+    | Ptree.Efor (_, _, _, _, _, _) | Ptree.Eassert (_, _) ->
+        (* Those cases can't be the resulting tuple to convert *)
+        e
+    | Ptree.Escope (a, e) -> add_loc @@ Ptree.Escope (a, convert e)
+    | Ptree.Eexn (a, b, c, e) -> add_loc @@ Ptree.Eexn (a, b, c, convert e)
+    | Ptree.Eoptexn (a, c, e) -> add_loc @@ Ptree.Eoptexn (a, c, convert e)
+    | Ptree.Ematch (e, brl, exnl) ->
+        let brl = List.map (fun (a,e) -> (a,convert e)) brl in
+        let exnl = List.map (fun (a,b,e) -> (a,b,convert e)) exnl in
+        add_loc @@ Ptree.Ematch (e, brl, exnl)
+    | Erec (a, e) -> add_loc @@ Erec(a, convert e)
+    | Elabel (a, e) -> add_loc @@ Elabel(a, convert e)
+    | Eattr (a, e) -> add_loc @@ Eattr(a, convert e)
+    | Ptree.Eghost e -> add_loc @@ Ptree.Eghost (convert e)
+    | Ecast (e, b) -> add_loc @@ Ecast (convert e, b)
+    | Esequence (e1, e2) -> add_loc @@ Esequence(e1,convert e2)
+    | Ptree.Eif  (e1, e2, e3) -> add_loc @@ Ptree.Eif(e1, convert e2, convert e3)
+    | Ptree.Elet (a, b, c, d, e) -> add_loc @@ Ptree.Elet (a,b,c,d,convert e)
+    | Erecord tl -> add_loc @@ convert_record e.expr_loc tl
+  in
+  let e = convert dwit in
+  let de = dexpr muc denv_empty e in
+  let ity = Ity.ity_tuple @@ List.map (fun (_,v) -> v.pv_ity) fl in
+  let dity = dity_of_ity ity in
+  let de = Dexpr.dexpr ?loc:de.de_loc (DEcast (de, dity)) in
+  let de = expr ~keep_loc:true ~ughost:false de in
+  de
+
+let add_is_field_attr id =
+  { id with id_ats = ATstr Ident.is_field_id_attr :: id.id_ats }
+
 let add_types muc tdl =
   let add m ({td_ident = {id_str = x}; td_loc = loc} as d) =
     Mstr.add_new (Loc.Located (loc, ClashSymbol x)) x d m in
@@ -1230,6 +1308,8 @@ let add_types muc tdl =
               let nms = Sstr.add_new exn nm nms in
               let ity = parse ~loc ~alias ~alg pty in
               let v = try Hstr.find hfd nm with Not_found ->
+                (* add proper attribute for later printing in dotted notation `t.id` *)
+                let id = add_is_field_attr id in
                 let v = create_pvsymbol (create_user_id id) ~ghost ity in
                 Hstr.add hfd nm v;
                 v in
@@ -1262,7 +1342,9 @@ let add_types muc tdl =
         let alg = Mstr.add x (id,args) alg in
         let get_fd nms fd =
           let {id_str = nm; id_loc = loc} = fd.f_ident in
-          let id = create_user_id fd.f_ident in
+          (* add proper attribute for later printing in dotted notation `t.id` *)
+          let id = add_is_field_attr fd.f_ident in
+          let id = create_user_id id in
           let ity = parse ~loc ~alias ~alg fd.f_pty in
           let ghost = d.td_vis = Abstract || fd.f_ghost in
           let pv = create_pvsymbol id ~ghost ity in
@@ -1285,21 +1367,7 @@ let add_types muc tdl =
             let gvars = List.fold_left add_fd Mstr.empty fl in
             let type_inv f = type_fmla_pure muc gvars Dterm.denv_empty f in
             let inv = List.map type_inv d.td_inv in
-            let add_w m (q,e) =
-              let v = try match q with
-                | Qident x -> Mstr.find x.id_str nms
-                | Qdot _ -> raise Not_found
-              with Not_found -> Loc.errorm ~loc:(qloc q)
-                "Unknown field %a" print_qualid q in
-              let dity = dity_of_ity v.pv_ity in
-              let de = dexpr muc denv_empty e in
-              let de = Dexpr.dexpr ?loc:de.de_loc (DEcast (de, dity)) in
-              Mpv.add v (expr ~keep_loc:true ~ughost:true de) m in
-            let wit = List.fold_left add_w Mpv.empty d.td_wit in
-            let wit = if d.td_wit = [] then [] else
-              List.map (fun (_,v) -> try Mpv.find v wit with
-                | _ -> Loc.errorm ?loc:v.pv_vs.vs_name.Ident.id_loc
-                  "Missing field %s" v.pv_vs.vs_name.id_string) fl in
+            let wit = Opt.map (type_wit muc nms fl) d.td_wit in
             let itd = create_plain_record_decl ~priv ~mut id args fl inv wit in
             Hstr.add hts x itd.itd_its; Hstr.add htd x itd
         end
@@ -1578,10 +1646,10 @@ let rec add_decl muc env file d =
       add_pdecl ~vc muc (create_exn_decl xs)
   | Ptree.Duseexport use ->
       use_export muc (find_module env file use)
-  | Ptree.Dcloneexport (use, inst) ->
+  | Ptree.Dcloneexport (loc, use, inst) ->
       let m = find_module env file use in
       warn_clone_not_abstract (qloc use) m.mod_theory;
-      clone_export muc m (type_inst muc m inst)
+      clone_export ~loc muc m (type_inst muc m inst)
   | Ptree.Duseimport (_loc,import,uses) ->
       let add_import muc (m, q) =
         let import = import || q = None in
@@ -1590,12 +1658,12 @@ let rec add_decl muc env file d =
         let muc = use_export muc m in
         close_scope muc ~import in
       List.fold_left add_import muc uses
-  | Ptree.Dcloneimport (_loc,import,qid,as_opt,inst) ->
+  | Ptree.Dcloneimport (loc,import,qid,as_opt,inst) ->
       let import = import || as_opt = None in
       let muc = open_scope muc (use_as qid as_opt).id_str in
       let m = find_module env file qid in
       warn_clone_not_abstract (qloc qid) m.mod_theory;
-      let muc = clone_export muc m (type_inst muc m inst) in
+      let muc = clone_export ~loc muc m (type_inst muc m inst) in
       let muc = close_scope muc ~import in
       muc
   | Ptree.Dimport q ->
