@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -49,7 +49,7 @@ let ident_printer =
   create_ident_printer bls ~sanitizer:san
 
 let print_ident fmt id =
-  fprintf fmt "%s" (id_unique ident_printer id)
+  pp_print_string fmt (id_unique ident_printer id)
 
 type info = {
   info_syn     : syntax_map;
@@ -67,7 +67,7 @@ let rec print_type info fmt ty = match ty.ty_node with
   | Tyapp (ts, l) ->
       begin match query_syntax info.info_syn ts.ts_name, l with
       | Some s, _ -> syntax_arguments s (print_type info) fmt l
-      | None, [] -> fprintf fmt "%a" print_ident ts.ts_name
+      | None, [] -> print_ident fmt ts.ts_name
       | None, _ ->
           begin match Mty.find_opt ty !(info.complex_type) with
           | Some ty -> print_type info fmt ty
@@ -85,7 +85,7 @@ let print_type info fmt ty = try print_type info fmt ty
   with Unsupported s -> raise (UnsupportedType (ty,s))
 
 let print_type_value info fmt = function
-  | None -> fprintf fmt "bool"
+  | None -> pp_print_string fmt "bool"
   | Some ty -> print_type info fmt ty
 
 (** var *)
@@ -93,7 +93,7 @@ let forget_var v = forget_id ident_printer v.vs_name
 
 let print_var fmt {vs_name = id} =
   let n = id_unique ident_printer id in
-  fprintf fmt "%s" n
+  pp_print_string fmt n
 
 let print_typed_var info fmt vs =
   fprintf fmt "%a::%a" print_var vs
@@ -115,7 +115,6 @@ let number_format = {
     Number.frac_real_support =
       `Custom
         ((fun fmt i -> pp_print_string fmt i),
-         (fun fmt i n -> fprintf fmt "(* %s %s)" i n),
          (fun fmt i n -> fprintf fmt "(/ %s %s)" i n));
   }
 
@@ -128,7 +127,7 @@ let rec print_term info fmt t = match t.t_node with
       | Some s -> syntax_arguments_typed s (print_term info)
         (print_type info) t fmt tl
       | None -> begin match tl with
-          | [] -> fprintf fmt "%a" print_ident ls.ls_name
+          | [] -> print_ident fmt ls.ls_name
           | _ -> fprintf fmt "@,(%a %a)"
               print_ident ls.ls_name (print_list space (print_term info)) tl
         end end
@@ -153,7 +152,7 @@ and print_fmla info fmt f = match f.t_node with
       | Some s -> syntax_arguments_typed s (print_term info)
         (print_type info) f fmt tl
       | None -> begin match tl with
-          | [] -> fprintf fmt "%a" print_ident ls.ls_name
+          | [] -> print_ident fmt ls.ls_name
           | _ -> fprintf fmt "(%a %a)"
               print_ident ls.ls_name (print_list space (print_term info)) tl
         end end
@@ -181,9 +180,9 @@ and print_fmla info fmt f = match f.t_node with
   | Tnot f ->
       fprintf fmt "@[(not@ %a)@]" (print_fmla info) f
   | Ttrue ->
-      fprintf fmt "true"
+      pp_print_string fmt "true"
   | Tfalse ->
-      fprintf fmt "false"
+      pp_print_string fmt "false"
   | Tif (f1, f2, f3) ->
       fprintf fmt "@[(if %a@ %a@ %a)@]"
         (print_fmla info) f1 (print_fmla info) f2 (print_fmla info) f3
