@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -129,7 +129,7 @@ val reload_files : ?hard_reload:bool -> controller -> bool * bool
     session returned.
 
   If parsing or typing errors occurs, a list of errors is raised
-  inside exception Errors_list.
+  inside exception {!Errors_list}.
 
   The detailed process of reloading the files of the given session is
   as follows.
@@ -142,25 +142,21 @@ val reload_files : ?hard_reload:bool -> controller -> bool * bool
 
   - each new theory is associated to a theory of the former session if
     the names match exactly. In case of no match:
-
-    . a new theory and its goals appear without any proof attempts in
-      it in the new session
-
-    . an unmatched old theory is kept in the new session together with
+    {ul {- a new theory and its goals appear without any proof attempts in
+         it in the new session}
+    {- an unmatched old theory is kept in the new session together with
       its former goals, proof attempts and transformations, but
-      without any tasks associated to goals and subgoals.
+      without any tasks associated to goals and subgoals.}}
 
   - within a new theory with a corresponding old theory, each goal is
     in turn associated to a former goal if possible. the match is done
     either on the goal name, or if no name match exactly, on the goal
     shape.
-
-    . a new goal without match is added with an empty set of proof
-      attempts and transformations
-
-    . an old goal without match is kept with all its former proof
+    {ul {- a new goal without match is added with an empty set of proof
+      attempts and transformations}
+    {- an old goal without match is kept with all its former proof
       attempts and transformations, but no task is associated to it,
-      neither to its subgoals.
+      neither to its subgoals.}}
 
   - on each new goal that has a matching old goal, old proof
     attempts are attached, with the status obsolete if the task has
@@ -170,10 +166,9 @@ val reload_files : ?hard_reload:bool -> controller -> bool * bool
     transformations are attached, and applied to the task, the
     generated subgoals are in turn matched to the old sub-goals, in
     the same manner as for goals in a theory
-
-    . an old sub-goals without a match is kept with all its former
+    {ul {- an old sub-goals without a match is kept with all its former
       proof attempts and transformations, but no task is associated to
-      it, neither to its subgoals.
+      it, neither to its subgoals.}}
 
   When the option [hard_reload] is true (false by default), libraries and
   drivers are also reloaded.
@@ -183,7 +178,7 @@ val reload_files : ?hard_reload:bool -> controller -> bool * bool
 val add_file : controller -> ?format:Env.fformat -> string -> unit
 (** [add_file cont ?fmt fname] parses the source file
     [fname] and add the resulting theories to the session of [cont].
-    parsing or typing errors are raised inside exception [Errors_list]
+    parsing or typing errors are raised inside exception {!Errors_list}
  *)
 
 val remove_subtree: notification:notifier -> removed:notifier ->
@@ -213,26 +208,28 @@ val register_notifier : (any -> unit) -> unit
  *)
 
 
-val interrupt : unit -> unit
+val interrupt : controller -> unit
 (** discards all scheduled proof attempts or transformations, including
     the ones already running *)
 
 val schedule_proof_attempt :
+  ?proof_script_filename:string ->
   controller ->
   proofNodeID ->
   Whyconf.prover ->
-  ?save_to:string ->
   limit:Call_provers.resource_limit ->
   callback:(proofAttemptID -> proof_attempt_status -> unit) ->
   notification:notifier -> unit
-(** [schedule_proof_attempt c id p ~timelimit ~callback ~notification] schedules a
+(** [schedule_proof_attempt ?proof_script_filename c id p ~timelimit
+   ~callback ~notification] schedules a
    proof attempt for a goal specified by [id] with the prover [p] with
    time limit [timelimit]; the function [callback] will be called each
    time the proof attempt status changes. Typically at Scheduled, then
    Running, then Done. If there is already a proof attempt with [p] it
    is updated.
-   [save_to] is used to give a location for the file generated for the prover
-   ( *.smt2). With debug flag keep_vcs, the file are saved at this location.
+   [proof_script_filename] is used to give a location for the files
+   generated for the prover.
+   With debug flag keep_vcs, the files are saved at this location.
 *)
 
 val schedule_edition :
