@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -143,11 +143,11 @@ let p s id =
   let pr = Ident.duplicate_ident_printer tables.Trans.printer in
   let apr = Ident.duplicate_ident_printer tables.Trans.aprinter in
   (* Use the external printer for exception reporting (default is identity) *)
-  (Pretty.create ~print_ext_any:(print_ext_any task lang) pr apr pr pr false)
+  (Pretty.create ~do_forget_all:false ~print_ext_any:(print_ext_any task lang) pr apr pr pr)
 
 let print_opt_type ~print_type fmt t =
   match t with
-  | None -> Format.fprintf fmt "bool"
+  | None -> Format.pp_print_string fmt "bool"
   | Some t -> print_type fmt t
 
 (* Exception reporting *)
@@ -171,7 +171,7 @@ let bypass_pretty s id =
   | Ty.UnboundTypeVar tv ->
       fprintf fmt "Unbound type variable: %a" P.print_tv tv
   | Ty.UnexpectedProp ->
-      fprintf fmt "Unexpected propositional type"
+      pp_print_string fmt "Unexpected propositional type"
   | Term.BadArity ({Term.ls_args = []} as ls, _) ->
       fprintf fmt "%s %a expects no arguments"
         (if ls.Term.ls_value = None then "Predicate" else "Function") P.print_ls ls
@@ -181,7 +181,7 @@ let bypass_pretty s id =
         (if ls.Term.ls_value = None then "Predicate" else "Function")
         P.print_ls ls i (if i = 1 then "" else "s") app_arg
   | Term.EmptyCase ->
-      fprintf fmt "Empty match expression"
+      pp_print_string fmt "Empty match expression"
   | Term.DuplicateVar vs ->
       fprintf fmt "Variable %a is used twice" P.print_vsty vs
   | Term.UncoveredVar vs ->
@@ -237,7 +237,7 @@ let bypass_pretty s id =
   | Decl.ClashIdent id ->
       fprintf fmt "Ident %s is defined twice" id.Ident.id_string
   | Decl.EmptyDecl ->
-      fprintf fmt "Empty declaration"
+      pp_print_string fmt "Empty declaration"
   | Decl.EmptyAlgDecl ts ->
       fprintf fmt "Algebraic type %a has no constructors" P.print_ts ts
   | Decl.EmptyIndDecl ls ->
@@ -301,7 +301,7 @@ let get_exception_message ses id e =
   | Generic_arg_trans_utils.Arg_trans_missing (s, svs) ->
       Pp.sprintf "Error in transformation function:\n%s %a\n" s
         (* The arguments should appear on one line (no @) *)
-        (Pp.print_list (fun fmt () -> fprintf fmt ", ") P.print_vs)
+        (Pp.print_list (fun fmt () -> pp_print_string fmt ", ") P.print_vs)
         (Term.Svs.elements svs),
       Loc.dummy_position, ""
   | Generic_arg_trans_utils.Arg_bad_hypothesis ("rewrite", _t) ->
@@ -323,11 +323,10 @@ let get_exception_message ses id e =
       Pp.sprintf "Parsing error: %a" Exn_printer.exn_printer e, loc, arg
   | Trans.Unnecessary_arguments l ->
       Pp.sprintf "First arguments were parsed and typed correctly but the last following are useless:\n%a"
-        (Pp.print_list Pp.newline (fun fmt s -> Format.fprintf fmt "%s" s)) l, Loc.dummy_position, ""
+        (Pp.print_list Pp.newline pp_print_string) l, Loc.dummy_position, ""
   | Generic_arg_trans_utils.Unnecessary_terms l ->
       Pp.sprintf "First arguments were parsed and typed correctly but the last following are useless:\n%a"
-        (Pp.print_list Pp.newline
-           (fun fmt s -> Format.fprintf fmt "%a" P.print_term s)) l, Loc.dummy_position, ""
+        (Pp.print_list Pp.newline P.print_term) l, Loc.dummy_position, ""
   | Args_wrapper.Arg_expected_none s ->
       Pp.sprintf "An argument was expected of type %s, none were given" s, Loc.dummy_position, ""
   | e ->
@@ -337,7 +336,7 @@ let get_exception_message ses id e =
 (* Command list *)
 (****************)
 
-let interrupt_query _cont _args = C.interrupt (); "interrupted"
+let interrupt_query cont _args = C.interrupt cont; "interrupted"
 
 let commands_table = Hstr.create 17
 
@@ -914,8 +913,10 @@ end
       let apr = tables.Trans.aprinter in
       (* For task printing we use the external printer (the default one is
          identity). *)
-      let module P = (val Pretty.create ~print_ext_any:(print_ext_any task lang) pr apr
-                         pr pr false) in
+      let module P =
+        (val Pretty.create ~print_ext_any:(print_ext_any task lang)
+               ~do_forget_all:false ~shorten_axioms:true
+               pr apr pr pr) in
       Pp.string_of (if show_full_context then P.print_task else P.print_sequent) task
     in
     task_text, loc_color_list, goal_loc
@@ -1005,18 +1006,8 @@ match pa.proof_state with
    begin
      let open Call_provers in
      let result = Pp.string_of print_prover_answer res.pr_answer in
-     let cont = d.cont in
-     let selected_model =
-       let th = Session_itp.find_th cont.controller_session parid in
-       let pm = Pmodule.restore_module (Theory.restore_theory (theory_name th)) in
-       let env = cont.controller_env in
-       let reduce_config =
-         let trans = "compute_in_goal" and prover = None in
-         Pinterp.rac_reduce_config_lit cont.controller_config env ~trans ?prover () in
-       let sel =
-         Counterexample.select_model ~check:true
-           ~reduce_config env pm res.pr_models in
-       match sel with None -> Model_parser.empty_model | Some (m,_) -> m in
+     let selected_model = Opt.get_def Model_parser.empty_model
+         (Check_ce.select_model_last_non_empty res.pr_models) in
      let ce_result =
        Pp.string_of (Model_parser.print_model_human ~filter_similar:true ~print_attrs ?me_name_trans:None)
          selected_model in
@@ -1028,7 +1019,7 @@ match pa.proof_state with
        let result_pr =
          result ^ "\n\n" ^ "Counterexample suggested by the prover:\n\n" ^ ce_result in
        let (source_result, list_loc, goal_loc, file_format) =
-         create_ce_tab cont.controller_session ~print_attrs
+         create_ce_tab d.cont.controller_session ~print_attrs
            selected_model any old_list_loc old_goal_loc in
        (P.notify (Source_and_ce (source_result, list_loc, goal_loc, file_format));
        P.notify (Task (nid, prover_text ^ result_pr, old_list_loc, old_goal_loc, lang)))
@@ -1210,7 +1201,7 @@ match pa.proof_state with
     | None -> P.notify (Message (Error "Please select a node id"))
     | Some any ->
         let unproven_goals = unproven_goals_below_id d.cont any in
-        List.iter (fun id -> C.schedule_proof_attempt ?save_to:None d.cont id
+        List.iter (fun id -> C.schedule_proof_attempt d.cont id
             prover ~limit ~callback ~notification:(notify_change_proved d.cont))
           unproven_goals
 
@@ -1423,7 +1414,7 @@ match pa.proof_state with
   let reload_session () : unit =
     let d = get_server_data () in
     (* interrupt all running provers and unfocus before reload *)
-    C.interrupt ();
+    C.interrupt d.cont;
     let _old_focus = !focused_node in
     focused_node := Unfocused;
     clear_tables ();
@@ -1648,7 +1639,7 @@ match pa.proof_state with
     | Get_task(nid,b,loc)          ->
        send_task nid b loc
     | Interrupt_req                ->
-       C.interrupt ()
+       C.interrupt d.cont
     | Reset_proofs_req             ->
        reset_proofs ();
        session_needs_saving := true
