@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -11,9 +11,6 @@
 
 open Itp_communication
 
-(** Useful functions for printing *)
-val convert_node_type_string: node_type -> string
-
 (** Print in Json format *)
 val print_request: Format.formatter -> ide_request -> unit
 val print_notification: Format.formatter -> notification -> unit
