@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -158,7 +158,7 @@ let print_id s tables =
   *)
   let pr = Ident.duplicate_ident_printer tables.Trans.printer in
   let apr = Ident.duplicate_ident_printer tables.Trans.aprinter in
-  let module P = (val Pretty.create pr apr pr pr false) in
+  let module P = (val Pretty.create ~do_forget_all:false pr apr pr pr) in
   (* Different constructs are printed differently *)
   match d.Decl.d_node, table_id with
   | Decl.Dind (_, il), Args_wrapper.Tsprsymbol pr ->
@@ -229,7 +229,7 @@ let search ~search_both s tables =
       *)
       let pr = Ident.duplicate_ident_printer tables.Trans.printer in
       let apr = Ident.duplicate_ident_printer tables.Trans.aprinter in
-      let module P = (val Pretty.create pr apr pr pr false) in
+      let module P = (val Pretty.create ~do_forget_all:false pr apr pr pr) in
       Pp.string_of (Pp.print_list Pp.newline2 P.print_decl) l
 
 let locate_id _cont task args =
@@ -257,7 +257,7 @@ let help_on_queries fmt commands =
   let l = Hstr.fold (fun c (h,_) acc -> (c,h)::acc) commands [] in
   let l = List.sort sort_pair l in
   let p fmt (c,help) = Format.fprintf fmt "%20s : %s" c help in
-  Format.fprintf fmt "%a" (Pp.print_list Pp.newline p) l
+  Pp.print_list Pp.newline p fmt l
 
 (* Return the prover corresponding to given name. name is of the form
   | name
