@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -96,6 +96,8 @@ val get_task : session -> proofNodeID -> Task.task
 val get_task_name_table : session -> proofNodeID -> Task.task * Trans.naming_table
 
 val get_transformations : session -> proofNodeID -> transID list
+val get_transformation : session -> proofNodeID -> string -> string list -> transID
+
 val get_proof_attempt_ids :
   session -> proofNodeID -> proofAttemptID Whyconf.Hprover.t
 
@@ -177,7 +179,7 @@ val add_file_section :
 (** [add_file_section s fn ths] adds a new
     'file' section in session [s], named [fn], containing fresh theory
     subsections corresponding to theories [ths]. The tasks of each
-    theory nodes generated are computed using [Task.split_theory].
+    theory nodes generated are computed using {!Task.split_theory}.
 
     Note that this function does not read anything from the file
     system. The file name [fn] is taken as is
@@ -216,12 +218,13 @@ val graft_proof_attempt : ?file:Sysutil.file_path -> session -> proofNodeID ->
 *)
 
 exception NoProgress
+
 val apply_trans_to_goal :
   allow_no_effect:bool -> session -> Env.env -> string -> string list ->
   proofNodeID -> Task.task list
 (** [apply_trans_to_goal s env tr args id] applies the transformation
   [tr] with arguments [args] to the goal [id], and returns the
-  subtasks.  raises [NoProgress] if [allow_no_effect] is false and [tr]
+  subtasks. Raises {!NoProgress} if [allow_no_effect] is false and [tr]
   returns the task unchanged *)
 
 val graft_transf : session -> proofNodeID -> string -> string list ->
@@ -267,7 +270,7 @@ val remove_subtree: notification:notifier -> removed:notifier ->
     called on each removed node, and notifier [notification] on nodes
     whose proved state changes.
 
-    raises [RemoveError] when removal is forbidden, e.g. when called on
+    Raises {!RemoveError} when removal is forbidden, e.g. when called on
     a file, a theory or a goal that is not detached
  *)
 
