@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -12,9 +12,11 @@
 (** {2 User-defined strategies} *)
 
 type instruction =
-  | Icall_prover of Whyconf.prover * int option * int option
+  | Icall_prover of Whyconf.prover * int option * int option * int option
     (** timelimit (if none use default timelimit),
-        memlimit (if none use default memlimit) *)
+        memlimit (if none use default memlimit)
+        steplimit (if none use no step limit)
+     *)
   | Itransform of string * int (** successor state on success *)
   | Igoto of int (** goto state *)
 
