@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -78,7 +78,7 @@ let option_list =
     "<file|dir> destination of extracted code";
   ]
 
-let config, env =
+let _config, env =
   Whyconf.Args.initialize option_list add_opt_file usage_msg
 
 let () =
@@ -351,11 +351,6 @@ let do_extract_module ?fname m =
   with Not_found -> () end;
   do_extract_module ?fname (Ident.Hid.mem visited_mod) m
 
-let find_pmod m mlw_file fname =
-  try Mstr.find m mlw_file with Not_found ->
-    eprintf "Module '%s' not found in the file '%s'.@." m fname;
-    exit 1
-
 let get_symbol ns find str_symbol =
   try let symbol = find ns [str_symbol] in Some symbol
   with Not_found -> None
