@@ -412,7 +412,7 @@ let comp t (state,task) = match t.task_decl.td_node with
       let old_keep_t = state.keep_t in
       let state = match dl with
         | _ when state.keep_t -> state
-        | [ts, [_]]
+        | [ts, [_,_::_]]
           when state.keep_r && not (Sid.mem ts.ts_name (get_decl_syms d)) ->
             { state with keep_t = true }
         | [{ ts_args = [] }, csl]
