@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -44,13 +44,13 @@ let add_literal (known_lit, decl as acc) t c ls_proj fin =
 let rec abstract_terms kn range_metas float_metas type_kept acc t =
   match t.t_node, t.t_ty with
   | Tapp (ls, [{ t_node = Tconst (Constant.ConstInt _ as c); t_ty = Some {ty_node = Tyapp (ts,[])} }]), _
-    when not (ts_equal ts ts_int || Sts.mem ts type_kept) && ls_equal ls (Mts.find ts range_metas) ->
+    when not (ts_equal ts ts_int || Sts.mem ts type_kept) && Opt.equal ls_equal (Some ls) (Mts.find_opt ts range_metas) ->
       acc, t_const c ty_int
   | Tapp (ls, [{ t_node = Tconst (Constant.ConstReal _ as c); t_ty = Some {ty_node = Tyapp (ts,[])} }]), _
     when not (ts_equal ts ts_real || Sts.mem ts type_kept) && ls_equal ls (fst (Mts.find ts float_metas)) ->
       acc, t_const c ty_real
   | Tconst (Constant.ConstInt _ as c), Some {ty_node = Tyapp (ts,[])}
-    when not (ts_equal ts ts_int || Sts.mem ts type_kept) ->
+    when not (ts_equal ts ts_int || Sts.mem ts type_kept) && Mts.mem ts range_metas ->
       let to_int = Mts.find ts range_metas in
       add_literal acc t c (Some to_int) None
   | Tconst (Constant.ConstReal _ as c), Some {ty_node = Tyapp (ts,[])}
