@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -19,14 +19,6 @@ open Task
 let intro_attr = create_attribute "introduced"
 let inline_trivial_attr = create_attribute "inline:trivial"
 
-let meta_get_counterexmp =
-  Theory.register_meta_excl "get_counterexmp" [Theory.MTstring]
-  ~desc:"Set@ when@ counter-example@ should@ be@ get."
-
-let get_counterexmp task =
-  let ce_meta = Task.find_meta_tds task meta_get_counterexmp in
-  not (Theory.Stdecl.is_empty ce_meta.tds_set)
-
 let rec relocate loc t =
   t_map (relocate loc) (t_attr_set ?loc t.t_attrs t)
 
@@ -95,8 +87,8 @@ let fold in_goal notls notdef task_hd (env, task) =
 let meta = Theory.register_meta "inline:no" [Theory.MTlsymbol]
   ~desc:"Disallow@ the@ inlining@ of@ the@ given@ function/predicate@ symbol."
 
-let t ?(use_meta=true) ?(in_goal=false) ~notls ~notdef =
-  Trans.bind (Trans.store get_counterexmp) (fun for_counterexample ->
+let t ~use_meta ~in_goal ~notls ~notdef =
+  Trans.bind (Trans.store Driver.get_counterexmp) (fun for_counterexample ->
     let trans notls =
       Trans.fold_map (fold in_goal notls notdef) Mls.empty None in
     if use_meta then
