@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -15,17 +15,11 @@ val intro_attr : Ident.attribute
 
 val meta : Theory.meta
 
-val get_counterexmp : Task.task -> bool
-(**
-   Returns true if counterexample should be get for the task.
-*)
-
-
 (** {2 Generic inlining} *)
 
 val t :
-  ?use_meta:bool ->
-  ?in_goal:bool ->
+  use_meta:bool ->
+  in_goal:bool ->
   notls:(for_counterexample:bool -> Term.lsymbol -> bool) ->
   notdef:(Term.term -> bool) ->
   Task.task Trans.trans
