@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -15,6 +15,7 @@
   for finding the right function in the Why3 API
 *)
 
+open Wstdlib
 open Ident
 open Ty
 open Term
@@ -90,8 +91,9 @@ let rec dequant ht pos f =
   | Tnot f1 ->
       t_not (dequant (not pos) f1)
   | Tlet (t,fb) ->
-      let vs, f1 = t_open_bound fb in
-      t_let_close vs t (dequant pos f1)
+     let vs, f1 = t_open_bound fb in
+     Hvs.replace ht vs (t_peek_bound fb);
+     (if pos then t_implies else t_and) (t_equ (t_var vs) t) (dequant pos f1)
   | Tcase (t,bl) ->
       let branch bf =
         let pat, f1 = t_open_branch bf in
@@ -137,11 +139,18 @@ let intro_var (subst, mal) (vs, id) =
   let subst = Mvs.add vs (fs_app ls [] vs.vs_ty) subst in
   (subst, mal), create_param_decl ls
 
-let get_expls f =
-  Sattr.filter (fun a -> Strings.has_prefix "expl:" a.attr_string) f.t_attrs
+let pushed_attr_prefixes = ref ["expl:"; "hyp_name:"]
 
-let get_hyp_names f =
-  Sattr.filter (fun a -> Strings.has_prefix "hyp_name:" a.attr_string) f.t_attrs
+let push_attributes_with_prefix s =
+  pushed_attr_prefixes := s :: !pushed_attr_prefixes
+
+let pushed_attrs f =
+  let add_by_prefix a pushed =
+    let has_prefix s = Strings.has_prefix s a.attr_string in
+    match List.find_opt has_prefix !pushed_attr_prefixes with
+    | Some s -> Mstr.add s a pushed
+    | None -> pushed in
+  Sattr.fold add_by_prefix f.t_attrs Mstr.empty
 
 (* Check if we already have a proposition with the same content.
    If we have a hash-consed one, and it is not yet used in the task,
@@ -159,16 +168,9 @@ let pr_of_premise f =
   let nm = Opt.get_def "H" nm in
   create_prsymbol (id_fresh nm ~attrs:intro_attrs)
 
-let rec intros kn pr axs mal (expl, hyp_name) f =
-  let aux_move attrs fattrs = (* attrs may be [expl] or [hyp_name] *)
-    let attrs = if Sattr.is_empty fattrs then attrs else fattrs in
-    let move f =
-      if Sattr.is_empty fattrs && not (Sattr.is_empty attrs)
-      then t_attr_add (Sattr.min_elt attrs) f else f in
-    attrs, move in
-  let expl, move_expl = aux_move expl (get_expls f) in
-  let hyp_name, move_hyp_name = aux_move hyp_name (get_hyp_names f) in
-  let move_attrs f = move_expl (move_hyp_name f) in
+let rec intros kn pr axs mal old_pushed f =
+  let pushed = Mstr.union (fun _ _ a -> Some a) old_pushed (pushed_attrs f) in
+  let move_attrs f = Mstr.fold (fun _ -> t_attr_add) pushed f in
   match f.t_node with
   (* (f2 \/ True) => _ *)
   | Tbinop (Timplies,{ t_node = Tbinop (Tor,f2,{ t_node = Ttrue }) },_)
@@ -199,7 +201,7 @@ let rec intros kn pr axs mal (expl, hyp_name) f =
         | Theory.MApr _ :: mal -> mal
         | _ -> mal in
       let _,axs,dl = List.fold_left add (Mvs.empty,axs,[]) fl in
-      List.rev_append dl (intros kn pr axs mal (expl, hyp_name) f2)
+      List.rev_append dl (intros kn pr axs mal pushed f2)
   | Tquant (Tforall,fq) ->
       let vsl,_trl,f_t = t_open_quant fq in
       let vsl = List.combine vsl (t_peek_quant fq) in
@@ -207,13 +209,13 @@ let rec intros kn pr axs mal (expl, hyp_name) f =
         Lists.map_fold_left intro_var (Mvs.empty, mal) vsl in
       (* preserve attributes and location of f  *)
       let f = t_attr_copy f (t_subst subst f_t) in
-      dl @ intros kn pr axs mal (expl, hyp_name) f
+      dl @ intros kn pr axs mal pushed f
   | Tlet (t,fb) ->
       let vs, f = t_open_bound fb in
       let ls, mal = ls_of_vs mal vs (t_peek_bound fb) in
       let f = t_subst_single vs (fs_app ls [] vs.vs_ty) f in
       let d = create_logic_decl [make_ls_defn ls [] t] in
-      d :: intros kn pr axs mal (expl, hyp_name) f
+      d :: intros kn pr axs mal pushed f
   | _ -> [create_prop_decl Pgoal pr (move_attrs f)]
 
 let intros kn mal pr f =
@@ -223,7 +225,7 @@ let intros kn mal pr f =
   let decls = Mtv.map create_ty_decl tvm in
   let subst = Mtv.map (fun ts -> ty_app ts []) tvm in
   let f = t_ty_subst subst Mvs.empty f in
-  let dl = intros kn pr Sdecl.empty mal (Sattr.empty, Sattr.empty) f in
+  let dl = intros kn pr Sdecl.empty mal Mstr.empty f in
   Mtv.values decls @ dl
 
 let rec introduce hd =
@@ -270,7 +272,8 @@ let rec generalize hd =
   | Theory.Decl {d_node = Dprop (Pgoal,pr,f)} ->
       let pl, task = apply_prev generalize hd in
       if pl = [] then [], Some hd else
-      let expl = get_expls f in
+      let pushed = pushed_attrs f in
+      let f = Mstr.fold (fun _ -> t_attr_remove) pushed f in
       let set_vs vs ls f =
         t_replace (t_app ls [] ls.ls_value) (t_var vs) f in
       let rewind (vl,f) d = match d.d_node with
@@ -289,17 +292,18 @@ let rec generalize hd =
         | _ -> assert false (* never *) in
       let vl, f = List.fold_left rewind ([],f) pl in
       let f = t_forall_close vl [] f in
-      let f = if Sattr.is_empty expl then f else
-              t_attr_add  (Sattr.min_elt expl) f in
+      let f = Mstr.fold (fun _ -> t_attr_add) pushed f in
       [], Task.add_decl task (create_prop_decl Pgoal pr f)
   | Theory.Decl ({d_node =
-      ( Dparam ({ls_args = []; ls_value = Some _} as ls)
-      | Dlogic [{ls_args = []; ls_value = Some _} as ls, _])} as d)
-    when Sattr.mem Inlining.intro_attr ls.ls_name.id_attrs ->
+      ( Dparam ({ls_args = []; ls_value = Some ty} as ls)
+      | Dlogic [{ls_args = []; ls_value = Some ty} as ls, _])} as d)
+    when Sattr.mem Inlining.intro_attr ls.ls_name.id_attrs &&
+         Stv.is_empty (Ty.ty_freevars Stv.empty ty) ->
       let pl, task = apply_prev generalize hd in
       d::pl, Task.add_meta task meta_intro_ls [Theory.MAls ls]
-  | Theory.Decl ({d_node = Dprop (Paxiom, pr, _)} as d)
-    when Sattr.mem Inlining.intro_attr pr.pr_name.id_attrs ->
+  | Theory.Decl ({d_node = Dprop (Paxiom, pr, ax)} as d)
+    when Sattr.mem Inlining.intro_attr pr.pr_name.id_attrs &&
+         Stv.is_empty (Term.t_ty_freevars Stv.empty ax) ->
       let pl, task = apply_prev generalize hd in
       d::pl, Task.add_meta task meta_intro_pr [Theory.MApr pr]
   (* We only reattach the local premises right before the goal.
@@ -323,7 +327,8 @@ let () = Trans.register_transform "generalize_introduced" generalize_intro
     Contributed by Nicolas Jeannerod [niols@niols.fr] *)
 let rec eliminate_exists_aux pr t =
   match t.t_node with
-  | Tquant (Texists, q) ->
+  | Tquant (Texists, q)
+    when Stv.is_empty (t_ty_freevars Stv.empty t) ->
      let vsl, _, t' = t_open_quant q in
      let intro_var subst vs =
        let id = id_clone ~attrs:intro_attrs vs.vs_name in
@@ -359,6 +364,9 @@ let simplify_intros =
 
 let split_vc =
   Trans.compose_l
+(*** do test:
+    (Trans.compose Simplify_formula.simplify_trivial_wp_quantification
+ *)
     (Trans.compose generalize_intro Split_goal.split_goal_right)
     (Trans.singleton simplify_intros)
 
