@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -35,3 +35,8 @@ val intros :
 val introduce_premises : Task.task Trans.trans
 
 val simplify_intros: Task.task Trans.trans
+
+val push_attributes_with_prefix : string -> unit
+(** [intros] preserves attributes with given prefixes (initially ["expl:"] and
+    ["hyp_name:"]) by pushing them through variables bindings. Custom attributes
+    prefixes can be added with this function. *)
