@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -30,7 +30,7 @@ let expl_reification_check = Ident.create_attribute "expl:reification check"
 
 let meta_decision_procedure =
   Theory.register_meta
-    ~desc:"decision procedure, used for reflection"
+    ~desc:"Declare a decision procedure, used for reflection."
     "reflection"
     [ Theory.MTid ]
 
