@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,9 +9,11 @@
 (*                                                                  *)
 (********************************************************************)
 
+open Mysexplib.Std_big_int [@@warning "-33"]
 open Big_int
 
 type t = big_int
+[@@deriving sexp_of]
 let compare = compare_big_int
 
 let zero = zero_big_int
@@ -37,8 +39,7 @@ let gt = gt_big_int
 let le = le_big_int
 let ge = ge_big_int
 
-let euclidean_div_mod x y =
-  if sign y = 0 then zero, zero else quomod_big_int x y
+let euclidean_div_mod x y = quomod_big_int x y
 
 let euclidean_div x y = fst (euclidean_div_mod x y)
 let euclidean_mod x y = snd (euclidean_div_mod x y)
@@ -67,3 +68,4 @@ let pow_int_pos_bigint = power_int_positive_big_int
 let to_string = string_of_big_int
 let of_string = big_int_of_string
 let to_int = int_of_big_int
+let is_int = is_int_big_int
