@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -10,7 +10,6 @@
 (********************************************************************)
 
 open Format
-open Wstdlib
 
 let char fmt = function
   | '"'  -> pp_print_string fmt "\\\""
@@ -28,39 +27,23 @@ let string fmt s =
   String.iter (char fmt) s;
   pp_print_char fmt '"'
 
-let int fmt d = fprintf fmt "%d" d
+let int = pp_print_int
 let bool fmt b = fprintf fmt "%b" b
-let standard_float fmt f = fprintf fmt "%f" f
 let float fmt f = fprintf fmt "%g" f
 
-let print_json_field key value_pr fmt value =
-  fprintf fmt "@[<hv 1>%a:@ %a@]" string key value_pr value
+let rec seq pr fmt = function
+  | [] -> ()
+  | [v] -> pr fmt v
+  | h::t ->
+      pr fmt h;
+      Format.fprintf fmt ",@ ";
+      seq pr fmt t
 
 let list pr fmt l =
-  if l = [] then fprintf fmt "[]"
-  else
-    fprintf fmt "@[<hv 1>%a@]"
-      (Pp.print_list_delim ~start:Pp.lsquare ~stop:Pp.rsquare ~sep:Pp.comma pr)
-      l
-
-let print_map_binding key_to_str value_pr fmt binding =
-  let (key, value) = binding in
-  print_json_field (key_to_str key) value_pr fmt value
-
-let map_bindings key_to_str value_pr fmt map_bindings =
-  if map_bindings = [] then fprintf fmt "{}"
-  else
-    fprintf fmt "@[<hv 1>%a@]"
-     (Pp.print_list_delim ~start:Pp.lbrace ~stop:Pp.rbrace ~sep:Pp.comma
-      (print_map_binding key_to_str value_pr)) map_bindings
-
-(* Convert a list of bindings into a map *)
-let convert_record l =
-  List.fold_left (fun m (k, x) -> Mstr.add k x m) Mstr.empty l
+  fprintf fmt "@[<hv 1>[%a]@]" (seq pr) l
 
 type json =
-  | Record of json Mstr.t
-  | Proj of json Mstr.t
+  | Record of (string * json) list
   | List of json list
   | String of string
   | Int of int
@@ -70,20 +53,27 @@ type json =
 
 let rec print_json fmt v =
   match v with
-  | Record r -> map_bindings (fun x -> x) print_json fmt (Mstr.bindings r)
-  | Proj p -> map_bindings (fun x -> x) print_json fmt (Mstr.bindings p)
+  | Record r ->
+      (* FIXME: sorting is only needed because tests are doing purely syntactic checks. *)
+      let r = List.sort (fun (k1,_) (k2,_) -> String.compare k1 k2) r in
+      map_bindings fmt r
   | List l -> list print_json fmt l
   | String s -> string fmt s
   | Int i -> int fmt i
   | Float f -> float fmt f
   | Bool b -> bool fmt b
-  | Null -> fprintf fmt "null"
+  | Null -> pp_print_string fmt "null"
+and map_binding fmt binding =
+  let (key, value) = binding in
+  fprintf fmt "@[<hv 1>%a:@ %a@]" string key print_json value
+and map_bindings fmt map_bindings =
+  fprintf fmt "@[<hv 1>{%a}@]" (seq map_binding) map_bindings
 
 
 (* Get json fields. Return Not_found if no fields or field missing *)
 let get_field j s =
   match j with
-  | Record r -> Mstr.find s r
+  | Record r -> List.assoc s r
   | _ -> raise Not_found
 
 let get_string j =
@@ -103,6 +93,7 @@ let get_list j =
 
 let get_float j =
   match j with
+  | Int n -> float_of_int n
   | Float f -> f
   | _ -> raise Not_found
 
@@ -115,3 +106,9 @@ let get_bool_opt j def =
   match j with
   | Bool b -> b
   | _ -> def
+
+let get_string_field j s = get_string (get_field j s)
+let get_int_field j s = get_int (get_field j s)
+let get_list_field j s = get_list (get_field j s)
+let get_float_field j s = get_float (get_field j s)
+let get_bool_field j s = get_bool (get_field j s)
