@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -111,6 +111,11 @@ let first_nth f l =
       | Some r -> n,r in
   aux f 0 l
 
+let rec drop_while p = function
+  | [] -> []
+  | a::l when p a -> drop_while p l
+  | l -> l
+
 let iteri f l =
   let rec iter i = function
     | [] -> ()
