@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -13,6 +13,7 @@
 type lexing_loc = Lexing.position * Lexing.position
 *)
 
+open Mysexplib.Std [@@warning "-33"]
 open Lexing
 
 let current_offset = ref 0
@@ -43,6 +44,7 @@ let report_line fmt l = fprintf fmt "%s:%d:" l.pos_fname l.pos_lnum
 *)
 
 type position = string * int * int * int
+[@@deriving sexp_of]
 
 let user_position fname lnum cnum1 cnum2 = (fname,lnum,cnum1,cnum2)
 
@@ -121,7 +123,7 @@ let () = Exn_printer.register
     | Located (loc,e) ->
         fprintf fmt "%a%a" report_position loc Exn_printer.exn_printer e
     | Message s ->
-        fprintf fmt "%s" s
+        pp_print_string fmt s
     | _ ->
         raise exn)
 
@@ -132,3 +134,39 @@ let with_location f lb =
     try f lb with
     | Located _ as e -> raise e
     | e -> raise (Located (loc lb, e))
+
+let get_line_lengths f =
+  let lengths = ref [] in
+  let ch = open_in f in
+  ( try
+      while true do
+        let line = input_line ch in
+        lengths := String.length line :: !lengths;
+      done
+    with End_of_file -> () );
+  close_in ch;
+  Array.of_list (List.rev !lengths)
+
+let get_line_lengths =
+  let module Str = struct include String let hash = Hashtbl.hash end in
+  let module Hstr = Hashtbl.Make (Str) in
+  let cache = Hstr.create 5 in
+  fun f ->
+    try Hstr.find cache f
+    with Not_found ->
+      let l = get_line_lengths f in
+      Hstr.add cache f l;
+      l
+
+let get_multiline (f, bl, bc, ec) =
+  if Sys.file_exists f then
+    let ls = get_line_lengths f in
+    let el, ec = ref bl, ref ec in
+    assert (!el > 0); (* lines are 1-indexed *)
+    while !el < Array.length ls && !ec > ls.(!el-1) do
+      ec := !ec - ls.(!el-1) - 1; (* newlines are counted in multi-line locs *)
+      incr el
+    done;
+    f, (bl, bc), (!el, !ec)
+  else
+    failwith "Loc.get_multiline"
