@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -24,6 +24,7 @@ val transfer_loc : Lexing.lexbuf -> Lexing.lexbuf -> unit
 (** {2 locations in files} *)
 
 type position
+[@@deriving sexp_of]
 
 val extract : Lexing.position * Lexing.position -> position
 val join : position -> position -> position
@@ -73,3 +74,10 @@ exception Message of string
 val errorm: ?loc:position -> ('a, Format.formatter, unit, 'b) format4 -> 'a
 
 val with_location: (Lexing.lexbuf -> 'a) -> (Lexing.lexbuf -> 'a)
+
+val get_multiline : position -> string * (int * int) * (int * int)
+(** Returns [filename, (bline, bcol), (eline, ecol)] of a position.
+
+    Currently, this function re-reads the file to determine the line and column of the end
+    of multiline positions. Subsequent calls for positions in the same file will not require
+    re-reading the file. The function fails if the file does not exist. *)
