@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -14,4 +14,7 @@
 (* Exception to be raised if mpfr is not installed *)
 exception Not_Implemented
 
-include Mpfr
+include Mlmpfr
+
+let get_formatted_str ?rnd ?base ?size mpfr_float =
+  get_formatted_str ?rnd ?base ?size mpfr_float
