@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -118,7 +118,8 @@ let rec basename p =
   | _ :: tl -> basename tl
 
 (* deprecated: let string_of_file_path p = String.concat "/" p *)
-let print_file_path fmt p = Format.fprintf fmt "%a" (Pp.print_list Pp.slash Pp.string) p
+let print_file_path fmt p =
+  Pp.print_list Pp.slash Pp.string fmt p
 
 
 let system_independent_path_of_file f =
@@ -160,7 +161,25 @@ let system_dependent_absolute_path dir p =
   let rec aux dir l =
     match l with
     | [] -> dir
-    | ".." :: xs when is_regular_dir dir -> aux (Filename.dirname dir) xs
+    | ".." :: xs when is_regular_dir dir ->
+        let d = Filename.basename dir in
+        let dir' = Filename.dirname dir in
+        if dir' = dir then
+          if dir = Filename.current_dir_name then
+            (* dir = "." *)
+            aux Filename.parent_dir_name xs
+          else
+            (* dir = "/" *)
+            aux dir' xs
+        else if d = Filename.current_dir_name then
+          (* dir = "foo/." *)
+          aux dir' l
+        else if d = Filename.parent_dir_name then
+          (* dir = "foo/.." *)
+          aux (Filename.dirname dir') l
+        else
+          (* dir = "foo/bar" *)
+          aux dir' xs
     | x :: xs -> aux (Filename.concat dir x) xs
   in
   aux dir p
