@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -35,19 +35,19 @@ exception FoldSkip
 
 val all_fn : ('a -> bool) -> 'z -> 'a -> bool
 (** [all_fn pr z a] return true if [pr a] is true,
-    otherwise raise FoldSkip *)
+    otherwise raise {!FoldSkip} *)
 
 val any_fn : ('a -> bool) -> 'z -> 'a -> bool
 (** [any_fn pr z a] return false if [pr a] is false,
-    otherwise raise FoldSkip *)
+    otherwise raise {!FoldSkip} *)
 
 val all2_fn : ('a -> 'b -> bool) -> 'z -> 'a -> 'b -> bool
 (** [all2_fn pr z a b] return true if [pr a b] is true,
-    otherwise raise FoldSkip *)
+    otherwise raise {!FoldSkip} *)
 
 val any2_fn : ('a -> 'b -> bool) -> 'z -> 'a -> 'b -> bool
 (** [any2_fn pr z a b] return false if [pr a b] is false,
-    otherwise raise FoldSkip *)
+    otherwise raise {!FoldSkip} *)
 
 type ('z,'a,'c) fold = ('z -> 'a -> 'z) -> 'z -> 'c -> 'z
 
@@ -71,6 +71,13 @@ val ffalse : 'a -> bool
 val ttrue : 'a -> bool
 (** [ttrue] constant function [true] *)
 
+(** [iter_first iter f] returns the first result of [f] that is inhabitated,
+    when applied on the elements encountered by iterator [iter]. Generalisation
+    of {!Lists.first}.
+
+    @raise Not_found if no such element is encountered by the iterator. *)
+val iter_first : (('a -> unit) -> 'b) -> ('a -> 'c option) -> 'c
+
 (** {3 Lexical comparison using projections}
 
     For example to lexically sort a list [l] of pairs [(int * string) list]:
