@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -16,9 +16,7 @@ let () = Debug.set_flag Glob.flag
 
 (* command line parsing *)
 
-let usage_msg = sprintf
-  "Usage: %s [options...] [files...]"
-  (Filename.basename Sys.argv.(0))
+let usage_msg = "<file>...\nProduce HTML documentation for the given files."
 
 let opt_output = ref None
 let opt_index = ref None (* default behavior *)
@@ -26,24 +24,23 @@ let opt_title = ref None
 let opt_body = ref false
 let opt_queue = Queue.create ()
 
-let option_list = [
-  "-o", Arg.String (fun s -> opt_output := Some s),
-      "<dir> print files in <dir>";
-  "--output", Arg.String (fun s -> opt_output := Some s),
-      " same as -o";
-  "--stdlib-url", Arg.String Doc_def.set_stdlib_url,
-      "<url> add links to <url> for files found on loadpath";
-  "--index", Arg.Unit (fun () -> opt_index := Some true),
+let option_list =
+  let open Getopt in
+  [ Key ('o', "output"), Hnd1 (AString, fun s -> opt_output := Some s),
+    "<dir> print files in <dir>";
+    KLong "stdlib-url", Hnd1 (AString, Doc_def.set_stdlib_url),
+    "<url> add links to <url> for files found on loadpath";
+    KLong "index", Hnd0 (fun () -> opt_index := Some true),
     " generate an index file index.html";
-  "--no-index", Arg.Unit (fun () -> opt_index := Some false),
+    KLong "no-index", Hnd0 (fun () -> opt_index := Some false),
     " do not generate an index file index.html";
-  "--title", Arg.String (fun s -> opt_title := Some s),
-    " <title> set a title for the index page";
-  "--body-only", Arg.Set opt_body,
+    KLong "title", Hnd1 (AString, fun s -> opt_title := Some s),
+    "<title> set a title for the index page";
+    KLong "body-only", Hnd0 (fun () -> opt_body := true),
     " only produce the body of the HTML document";
-]
+  ]
 
-let _,_,env =
+let _,env =
   Whyconf.Args.initialize option_list
     (fun x -> Queue.add x opt_queue) usage_msg
 
@@ -118,10 +115,3 @@ let () =
   with e when not (Debug.test_flag Debug.stack_trace) ->
     eprintf "%a@." Exn_printer.exn_printer e;
     exit 1
-
-(*
-Local Variables:
-compile-command: "unset LANG; make -C ../.. bin/why3doc.opt"
-End:
-*)
-
