@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -29,20 +29,16 @@ let add_element s =
   opt_elements := Some l
 
 let spec =
-  ("-style",
-   Arg.Set_int opt_style,
-   "<n> set output style (1 or 2, default 1)") ::
-  ("-o",
-   Arg.Set_string opt_output_dir,
-   "<dir> where to produce LaTeX files (default: session dir)") ::
-  ("-e",
-   Arg.String add_element,
-   "<path> produce a table for the element denoted by <path>") ::
-  ("-longtable",
-   Arg.Set opt_longtable,
-   " use 'longtable' environment instead of 'tabular'") ::
-  common_options
-
+  let open Getopt in
+  [ KLong "style", Hnd1 (AInt, fun i -> opt_style := i),
+    "<n> set output style (1 or 2, default 1)";
+    KShort 'o', Hnd1 (AString, fun s -> opt_output_dir := s),
+    "<dir> set output directory (default: session dir)";
+    KShort 'e', Hnd1 (AString, add_element),
+    "<path> produce a table for the given element";
+    KLong "longtable", Hnd0 (fun () -> opt_longtable := true),
+    " use 'longtable' environment instead of 'tabular'"
+  ]
 
 (* Statistics in LaTeX*)
 
@@ -94,10 +90,9 @@ let print_head n depth provers fmt =
   fprintf fmt "\\\\ @."
 
 let print_tabular_head n depth provers fmt =
-  fprintf fmt "\\begin{tabular}";
-  fprintf fmt "{|l|";
-  for _i = 0 to depth do fprintf fmt "l|" done;
-  for _i = 1 to (List.length provers) do fprintf fmt "c|" done;
+  pp_print_string fmt "\\begin{tabular}{|l|";
+  for _i = 0 to depth do pp_print_string fmt "l|" done;
+  for _i = 1 to (List.length provers) do pp_print_string fmt "c|" done;
   fprintf fmt "}@.";
   print_head n depth provers fmt
 
@@ -125,17 +120,17 @@ let print_result_prov s proofs prov fmt=
                   fprintf fmt "& \\outofmemory{%dM} "
                     pr.limit.Call_provers.limit_mem
 		| Call_provers.StepLimitExceeded ->
-                  fprintf fmt "& \\steplimitexceeded "
+                  pp_print_string fmt "& \\steplimitexceeded "
 		| Call_provers.Unknown _ ->
                   fprintf fmt "& \\unknown{%.2f} " res.Call_provers.pr_time
 		| Call_provers.Failure _ ->
-                  fprintf fmt "& \\failure "
+                  pp_print_string fmt "& \\failure "
 		| Call_provers.HighFailure ->
-                  fprintf fmt "& \\highfailure "
+                  pp_print_string fmt "& \\highfailure "
 
 	    end
-	| None -> fprintf fmt "(no result)"
-  with Not_found -> fprintf fmt "& \\noresult") prov;
+	| None -> pp_print_string fmt "(no result)"
+  with Not_found -> pp_print_string fmt "& \\noresult") prov;
   fprintf fmt "\\\\ @."
 
 
@@ -154,24 +149,24 @@ let rec goal_latex_stat s fmt prov depth depth_max subgoal g =
 	  begin
 	    if(depth < depth_max)  then
 	      for _i = 1 to depth do
-                fprintf fmt " & "
+                pp_print_string fmt " & "
               done
 	    else
 	      for _i = 1 to depth - 1 do
-                fprintf fmt " & "
+                pp_print_string fmt " & "
               done
 	  end
 	else
 	  if(depth < depth_max) then
 	    if depth > 0 then
-              fprintf fmt " & "
+              pp_print_string fmt " & "
       end
     else
       begin
 	if subgoal > 0  then
-	  for _i = 1 to depth do fprintf fmt " & " done
+	  for _i = 1 to depth do pp_print_string fmt " & " done
 	else
-	  if depth > 0 then fprintf fmt " & "
+	  if depth > 0 then pp_print_string fmt " & "
       end;
     if (depth <= 1) then
       fprintf fmt "\\explanation{%s} "
@@ -217,7 +212,7 @@ let style_2_row fmt ?(transf=false) depth prov subgoal expl=
     fprintf fmt "\\cline{%d-%d}@." 2 column
   else
     fprintf fmt "\\hline@.";
-  for _i = 1 to depth do fprintf fmt "\\quad" done;
+  for _i = 1 to depth do pp_print_string fmt "\\quad" done;
   let macro = if transf then "transformation" else "explanation" in
   if depth = 0 || transf then
     fprintf fmt "\\%s{%s} " macro expl
@@ -288,9 +283,8 @@ let latex_tabular_file n s fmt depth provers f =
 
 
 let latex_longtable n s fmt depth name provers t=
-  fprintf fmt "\\begin{longtable}";
-  fprintf fmt "{| l |";
-  for _i = 0 to (List.length provers) + depth do fprintf fmt "c |" done;
+  pp_print_string fmt "\\begin{longtable}{| l |";
+  for _i = 0 to (List.length provers) + depth do pp_print_string fmt "c |" done;
   fprintf fmt "}@.";
   (* First head *)
   print_head n depth provers fmt;
@@ -448,7 +442,7 @@ let print_latex_statistics n table dir session =
 let table () = if !opt_longtable then "longtable" else "tabular"
 
 let run_one fname =
-  let ses,_ = read_session fname in
+  let ses = read_session fname in
   let project_dir = get_dir ses in
   let dir = if !opt_output_dir = "" then project_dir else
       !opt_output_dir
@@ -456,8 +450,7 @@ let run_one fname =
   print_latex_statistics !opt_style (table ()) dir ses
 
 let run () =
-  let _,_,should_exit1 = read_env_spec () in
-  if should_exit1 then exit 1;
+  let _,_ = Whyconf.Args.complete_initialization () in
   iter_files run_one
 
 
